/***************************************************************************
                          cobjecttree.cpp  -  description
                             -------------------
    begin                : Mon May 7 2001
    copyright            : (C) 2001 by Mihnea Balta
    email                : mihnea@tomrad.ro
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cobjecttree.h"
#include "dbdesign.h"
#include "sequence.h"

CObjectTree::CObjectTree(QWidget *parent, const char *name ) : KListView(parent,name) {
  QPixmap table_ico( ( const char** ) table_xpm );
  QPixmap relation_ico( ( const char** ) relation_xpm );
  QPixmap function_ico( ( const char** ) function_xpm );
  QPixmap sequence_ico( ( const char** ) sequence_xpm );
	setSorting(-1);
	addColumn("Items");

  tblroot = new CObjTreeItem((void *)0, TABLE_ROOT, this);
  tblroot->setText(0, "Tables");
  tblroot->setPixmap(0, table_ico);

  relroot = new CObjTreeItem((void *)0, REL_ROOT, this, tblroot);
  relroot->setText(0, "Relations");
  relroot->setPixmap(0, relation_ico);

  funcroot = new CObjTreeItem((void *)0, FUNC_ROOT, this, relroot);
  funcroot->setText(0, "Functions");
  funcroot->setPixmap(0, function_ico);

  seqroot = new CObjTreeItem((void *)0, SEQ_ROOT, this, funcroot);
  seqroot->setText(0, "Sequences");
  seqroot->setPixmap(0, sequence_ico);
	
	setFrameShape( QListView::StyledPanel );
  setMouseTracking(false);
  setRootIsDecorated(true);
	
	DbDesign *main = (DbDesign *)(parent);
	tableMenu = new QPopupMenu(this);
	tbl_menu_id[0] = tableMenu->insertItem("");
	tableMenu->insertSeparator();
	tbl_menu_id[1] = tableMenu->insertItem("&New table", main->dbview, SLOT(slotNewTbl()));
	tbl_menu_id[2] = tableMenu->insertItem("&Edit table", main->dbview, SLOT(slotEditTbl()));
	tbl_menu_id[3] = tableMenu->insertItem("&Delete table", main->dbview, SLOT(slotDelTbl()));
	tbl_menu_id[4] = tableMenu->insertItem("E&xport to...", main->dbview, SLOT(slotExportTbl()));
	
	relMenu = new QPopupMenu(this);
	rel_menu_id[0] = relMenu->insertItem("");
	relMenu->insertSeparator();
	rel_menu_id[1] = relMenu->insertItem("&New relation", main->dbview, SLOT(slotNewRel()));
	rel_menu_id[2] = relMenu->insertItem("&Edit relation", main->dbview, SLOT(slotEditRel()));
	rel_menu_id[3] = relMenu->insertItem("&Delete relation", main->dbview, SLOT(slotDelRel()));

	seqMenu = new QPopupMenu(this);
	seq_menu_id[0] = seqMenu->insertItem("");
	seqMenu->insertSeparator();
	seq_menu_id[1] = seqMenu->insertItem("&New sequence", main->dbview, SLOT(slotNewSeq()));
	seq_menu_id[2] = seqMenu->insertItem("&Edit sequence", main->dbview, SLOT(slotEditSeq()));
	seq_menu_id[3] = seqMenu->insertItem("&Delete sequence", main->dbview, SLOT(slotDelSeq()));
}

CObjectTree::~CObjectTree(){
}

void CObjectTree::contentsMousePressEvent(QMouseEvent *e){
	KListView::contentsMousePressEvent(e);
	CObjTreeItem *i = (CObjTreeItem *)(itemAt(e->pos()));
	if(!i)
		return;
	//tables
	if(i->getType() == TABLE_IT){
		DbDesign *main = (DbDesign *)(parent());
		main->dbview->setFocusItem((CTableItem *)i->getObject());
		if(e->button() == RightButton){
   		QString s;
   		CTableItem *ti = (CTableItem *)i->getObject();
			s.sprintf("-= %s =-", ti->getTable()->name());
			tableMenu->changeItem(tbl_menu_id[0], s);
			for(int i=2; i<5; i++)
				tableMenu->setItemEnabled(tbl_menu_id[i], true);
			QPoint p = mapToGlobal(e->pos());
			p.setX(p.x()+2);
			tableMenu->popup(p);
		}
	}
	if( (i->getType() == TABLE_ROOT) && (e->button() == RightButton) ){
		tableMenu->changeItem(tbl_menu_id[0], "-= Tables =-");
		for(int i=2; i<5; i++)
			tableMenu->setItemEnabled(tbl_menu_id[i], false);
		QPoint p = mapToGlobal(e->pos());
		p.setX(p.x()+2);
		tableMenu->popup(p);
	}
	
	//relations
	if(i->getType() == REL_IT){
		DbDesign *main = (DbDesign *)(parent());
		main->dbview->setFocusRel((CRelationItem *)i->getObject());
		if(e->button() == RightButton){
   		QString s;
   		CRelationItem *ti = (CRelationItem *)i->getObject();
			s.sprintf("-= %s =-", ti->getRel()->name.latin1());
			relMenu->changeItem(rel_menu_id[0], s);
			for(int i=2; i<5; i++)
				relMenu->setItemEnabled(rel_menu_id[i], true);
			QPoint p = mapToGlobal(e->pos());
			p.setX(p.x()+2);
			relMenu->popup(p);
		}
	}
	if( (i->getType() == REL_ROOT) && (e->button() == RightButton) ){
		relMenu->changeItem(rel_menu_id[0], "-= Relations =-");
		for(int i=2; i<5; i++)
			relMenu->setItemEnabled(rel_menu_id[i], false);
		QPoint p = mapToGlobal(e->pos());
		p.setX(p.x()+2);
		relMenu->popup(p);
	}
	
	//sequences
	if(i->getType() == SEQ_IT){
		DbDesign *main = (DbDesign *)(parent());
		main->dbview->setFocusSeq((CSequence *)i->getObject());
		if(e->button() == RightButton){
   		QString s;
   		CSequence *ti = (CSequence *)i->getObject();
			s.sprintf("-= %s =-", ti->name.latin1());
			seqMenu->changeItem(seq_menu_id[0], s);
			for(int i=2; i<5; i++)
				seqMenu->setItemEnabled(seq_menu_id[i], true);
			QPoint p = mapToGlobal(e->pos());
			p.setX(p.x()+2);
			seqMenu->popup(p);
		}
	}
	if( (i->getType() == SEQ_ROOT) && (e->button() == RightButton) ){
		seqMenu->changeItem(seq_menu_id[0], "-= Sequences =-");
		for(int i=2; i<5; i++)
			seqMenu->setItemEnabled(seq_menu_id[i], false);
		QPoint p = mapToGlobal(e->pos());
		p.setX(p.x()+2);
		seqMenu->popup(p);
	}
}

void CObjectTree::contentsMouseDoubleClickEvent(QMouseEvent *e){
	KListView::contentsMouseDoubleClickEvent(e);
	CObjTreeItem *i = (CObjTreeItem *)(itemAt(e->pos()));
	if(!i)
		return;
	if(i->getType() == TABLE_IT){
		DbDesign *main = (DbDesign *)(parent());
		main->dbview->setFocusItem((CTableItem *)i->getObject());
		main->dbview->slotEditTbl();
		return;
	}
	if(i->getType() == REL_IT){
		DbDesign *main = (DbDesign *)(parent());
		main->dbview->setFocusRel((CRelationItem *)i->getObject());
		main->dbview->slotEditRel();
		return;
	}
	if(i->getType() == FUNC_IT){
		//TODO
	}
	if(i->getType() == SEQ_IT){
		DbDesign *main = (DbDesign *)(parent());
		main->dbview->setFocusSeq((CSequence *)i->getObject());
		main->dbview->slotEditSeq();
		return;
	}
}

void CObjectTree::insTableLast(const char* name, CTableItem *tableref){
	CObjTreeItem *li;
	CObjTreeItem *last = (CObjTreeItem *)(tblroot->firstChild());
	void *p = tableref;
	if(last!=0){
		for(;last->nextSibling()!=0;last=(CObjTreeItem *)(last->nextSibling()));
		li = new CObjTreeItem(p, TABLE_IT, tblroot, last);
	}
	else
		li = new CObjTreeItem(p, TABLE_IT, tblroot);
	li->setText( 0, QString(name) );
}

void CObjectTree::chgTableItem(const char* oldn, const char* newn){
	CObjTreeItem *li = tblroot;
	for(li = (CObjTreeItem *)(li->firstChild()); li!=0; li = (CObjTreeItem *)(li->nextSibling()))
		if(li->text(0) == oldn){
			li->setText(0, newn);
			break;
		}
}

void CObjectTree::delTableItem(const char* name){
	for(CObjTreeItem *li = (CObjTreeItem *)tblroot->firstChild(); li!=0; li = (CObjTreeItem *)li->nextSibling())
		if(li->text(0) == name){
			delete li;
			break;
		}
}

void CObjectTree::insRelLast(QString &name, CRelationItem *relref){
	CObjTreeItem *li;
	CObjTreeItem *last = (CObjTreeItem *)(relroot->firstChild());
	void *p = relref;
	if(last!=0){
		for(;last->nextSibling()!=0;last=(CObjTreeItem *)(last->nextSibling()));
		li = new CObjTreeItem(p, REL_IT, relroot, last);
	}
	else
		li = new CObjTreeItem(p, REL_IT, relroot);
	li->setText( 0, QString(name) );
}

void CObjectTree::chgRelItem(QString& oldn, QString& newn){
	CObjTreeItem *li = relroot;
	for(li = (CObjTreeItem *)(li->firstChild()); li!=0; li = (CObjTreeItem *)(li->nextSibling()))
		if(li->text(0) == oldn){
			li->setText(0, newn);
			break;
		}
}

void CObjectTree::delRelItem(QString &s){
	for(CObjTreeItem *li = (CObjTreeItem *)relroot->firstChild(); li!=0; li = (CObjTreeItem *)li->nextSibling())
		if(li->text(0) == s){
			delete li;
			break;
		}
}

void CObjectTree::insSeqLast(CSequence *item){
	CObjTreeItem *li;
	CObjTreeItem *last = (CObjTreeItem *)(seqroot->firstChild());
	void *p = item;
	if(last!=0){
		for(;last->nextSibling()!=0;last=(CObjTreeItem *)(last->nextSibling()));
		li = new CObjTreeItem(p, SEQ_IT, seqroot, last);
	}
	else
		li = new CObjTreeItem(p, SEQ_IT, seqroot);
	li->setText( 0, item->name );
}

void CObjectTree::chgSeqItem(QString& oldn, QString& newn){
	CObjTreeItem *li = seqroot;
	for(li = (CObjTreeItem *)(li->firstChild()); li!=0; li = (CObjTreeItem *)(li->nextSibling()))
		if(li->text(0) == oldn){
			li->setText(0, newn);
			break;
		}
}

void CObjectTree::delSeqItem(QString &s){
	for(CObjTreeItem *li = (CObjTreeItem *)seqroot->firstChild(); li!=0; li = (CObjTreeItem *)li->nextSibling())
		if(li->text(0) == s){
			delete li;
			break;
		}
}
