/***************************************************************************
                          cobjecttree.h  -  description
                             -------------------
    begin                : Mon May 7 2001
    copyright            : (C) 2001 by Mihnea Balta
    email                : mihnea@tomrad.ro
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COBJECTTREE_H
#define COBJECTTREE_H

#include <qwidget.h>
#include <klistview.h>
#include <qpopupmenu.h>
#include <kmessagebox.h>

#include "cobjtreeitem.h"
#include "ctableitem.h"
#include "crelationitem.h"
#include "creledit.h"

/****
  @author Mihnea Balta
  This is the object tree that appears on the left of the main widget.
****/

static const char* const table_xpm[] = {
"16 16 4 1",
"a c #00c200",
"b c #00ff00",
". c #c5ffc5",
"# c #ff0000",
"................",
".##############.",
".#aaa#aaaa#aaa#.",
".#aaa#aaaa#aaa#.",
".##############.",
".#bbb#bbbb#bbb#.",
".#bbb#bbbb#bbb#.",
".#bbb#bbbb#bbb#.",
".#bbb#bbbb#bbb#.",
".#bbb#bbbb#bbb#.",
".#bbb#bbbb#bbb#.",
".#bbb#bbbb#bbb#.",
".#bbb#bbbb#bbb#.",
".#bbb#bbbb#bbb#.",
".##############.",
"................"};

static const char* const relation_xpm[] = {
"16 16 2 1",
". c None",
"# c #ff0000",
"................",
"................",
"................",
"................",
"...#........#...",
"..##........##..",
".##############.",
"################",
".##############.",
"..##........##..",
"...#........#...",
"................",
"................",
"................",
"................",
"................"};

static const char* const function_xpm[] = {
"16 16 2 1",
". c None",
"# c #000000",
"................",
".....#..........",
"....##..........",
"....#...........",
"...#....#..#....",
"...#....#..#....",
"...#...#....#...",
"..###..#....#...",
"...#...#....#...",
"...#...#....#...",
"...#...#....#...",
"...#...#....#...",
"..#.....#..#....",
".##.....#..#....",
".#..............",
"................"};

static const char* const sequence_xpm[] = {
"16 16 2 1",
". c None",
"# c #000000",
"................",
"................",
"................",
"................",
"...#...###......",
"..##.....#......",
"..##.....#......",
".#.#.....#......",
"...#...###......",
"...#...#........",
"...#...#........",
"...#...#........",
"..####.####.#.#.",
".....#....#.....",
"....#....#......",
"................"};

class CObjectTree : public KListView  {
   Q_OBJECT
public: 
	CObjectTree(QWidget *parent=0, const char *name=0);
	~CObjectTree();
	void	insTableLast(const char*, CTableItem*);
	void	chgTableItem(const char*, const char*);
	void	delTableItem(const char*);
	void	insRelLast(QString&, CRelationItem*);
	void	chgRelItem(QString&, QString&);
	void	delRelItem(QString&);
	void	insSeqLast(CSequence*);
	void	chgSeqItem(QString&, QString&);
	void	delSeqItem(QString&);
	
protected:
	void contentsMousePressEvent(QMouseEvent *e);
	void contentsMouseDoubleClickEvent(QMouseEvent *e);
private:
	QPopupMenu	*tableMenu;
	QPopupMenu	*relMenu;
	QPopupMenu	*seqMenu;
	int					tbl_menu_id[5];
	int					rel_menu_id[4];
	int					seq_menu_id[4];
	CObjTreeItem	*tblroot;
	CObjTreeItem	*relroot;
	CObjTreeItem	*funcroot;
	CObjTreeItem	*seqroot;
};

#endif
