/***************************************************************************
                          crelationitem.cpp  -  description
                             -------------------
    begin                : Mon May 7 2001
    copyright            : (C) 2001 by Mihnea Balta
    email                : mihnea@tomrad.ro
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "crelationitem.h"
#include <iostream>
#include <math.h>
#include "fldcolors.h"

CRelationItem::CRelationItem(CRelation *relref, QCanvas *canvas) : QCanvasPolygon(canvas){
	rel = relref;
	view = (CView*)canvas->parent();
	QPointArray pa(4);
	pa.putPoints(0, 4, 0, 0, 1, 1, 2, 2, 1, 1);
	setPoints(pa);
	visible = true;
}

CRelationItem::~CRelationItem(){
	delete rel;
}

void CRelationItem::setRel(CRelation *newrel){
	delete rel;
	rel = newrel;
}

inline float dist(QPoint& p1, QPoint& p2){
	return sqrt( (p1.x()-p2.x())*(p1.x()-p2.x()) + (p1.y()-p2.y())*(p1.y()-p2.y()) );
}

#define inf 1000000.0f

inline float angle(QPoint& a, QPoint& b, QPoint& c){
	float t1, t2;
	float x1 = a.x();
	float x2 = b.x();
	float x3 = c.x();
	float y1 = a.y();
	float y2 = b.y();
	float y3 = c.y();
	
	if(x1 != x2)
		t1 = fabs( (y1 - y2) / (x1 - x2) );
	else
		t1 = inf;
	if(x1 != x3)
		t2 = fabs( (y1 - y3) / (x1 - x3) );
	else
		t2 = inf;
	return fabs(t1 - t2);
}

inline int min(int a, int b){
	if (a<b)
		return a;
	else
		return b;
}

inline int max(int a, int b){
	if (a>b)
		return a;
	else
		return b;
}

int iswound(QPoint& p1, QPoint& p2, QPoint& p3, QPoint& p4){
	float a = dist(p1, p2) + dist(p3, p4);
	float b = dist(p1, p3) + dist(p2, p4);
	return (a>b);
}

void CRelationItem::drawShape(QPainter &p){
	if(!visible)
		return;
	CTableItem *from = view->getTblItemfromId(rel->fromTbl);
	CTableItem *to = view->getTblItemfromId(rel->toTbl);
	int x1 = int(from->x());
	int y1 = int(from->y());
	int x2 = int(to->x());
	int y2 = int(to->y());
	int w1 = int(from->width());
	int h1 = int(from->height());
	int w2 = int(to->width());
	int h2 = int(to->height());
	QPoint f1, f2, t1, t2;
	QPointArray a(4), b(4);
	int i, j, k=0, l=0;
	int fx, fy, tx, ty, sz;
	float m;

	a[0] = QPoint(x1, y1);
	a[1] = QPoint(x1, y1+h1);
	a[2] = QPoint(x1+w1, y1+h1);
	a[3] = QPoint(x1+w1, y1);
	
	b[0] = QPoint(x2, y2);
	b[1] = QPoint(x2, y2+h2);
	b[2] = QPoint(x2+w2, y2+h2);
	b[3] = QPoint(x2+w2, y2);
	
//	cerr << "Entering drawShape() for '" << rel->name << "'" << endl;
	/****
	the following piece of code finds the segments (f1, f2)
	and (t1, t2) which hopefully represent one side of each
	of the CTableItems (from and to, respectively) and (still
	hopefully) are the best to link
	****/
	m = 100000;
	for(i=0; i<4; i++)
		for(j=0; j<4; j++)
			if(dist(a[i], b[j])<m){
				m = dist(a[i], b[j]);
				f1 = a[i];
				t1 = b[j];
				k = j;
				l = i;
			}
	m = 0;
	for(i=0; i<4; i++)
		if( (i!=k) && (angle(f1, t1, b[i]) > m) ){
			t2 = b[i];
			m = angle(f1, t1, b[i]);
		}
	m = 100000;
	for(i=0; i<4; i++)
		if( (i!=l) && (dist(t2, a[i]) < m) ){
			f2 = a[i];
			m = dist(t2, a[i]);
		}
	
	sz = 0;
	fldlist fl = from->getTable()->getFields();
	for(fldlist::iterator it = fl.begin(); it != fl.end(); it++){
		if( (it->flags & FIELD_FK) || (it->refcnt > 0) )
			sz++;
		if(it->getId() == rel->fromFld)
			i = sz-1;
	}
	k = i;
	
	fx = int( fabs( floor( (f1.x() - f2.x()) / sz ) ) );
	fy = int( fabs( floor( (f1.y() - f2.y()) / sz ) ) );
	fx = min(f1.x(), f2.x()) + fx * i + int(floor(fx/2));
	fy = min(f1.y(), f2.y()) + fy * i + int(floor(fy/2));
	
	tx = int( fabs( floor( (t1.x() - t2.x()) / sz ) ) );
	ty = int( fabs( floor( (t1.y() - t2.y()) / sz ) ) );
	tx = min(t1.x(), t2.x()) + tx * i + int(floor(tx/2));
	ty = min(t1.y(), t2.y()) + ty * i + int(floor(ty/2));
	
	QPointArray pa(8);
	QPoint p1, p2, p3, p4, p5, p6, p7, p8;
	if(f1.y() == f2.y())
		if(f1.y() == from->y()){
			p1.setX(fx-2);
			p1.setY(fy);
			p2.setX(fx-2);
			p2.setY(fy-4);
			p3.setX(fx+2);
			p3.setY(fy-4);
			p4.setX(fx+2);
			p4.setY(fy);
		}
		else{
			p1.setX(fx-2);
			p1.setY(fy);
			p2.setX(fx-2);
			p2.setY(fy+4);
			p3.setX(fx+2);
			p3.setY(fy+4);
			p4.setX(fx+2);
			p4.setY(fy);
		}
	else
		if(f1.x() == from->x()){
			p1.setX(fx);
			p1.setY(fy-2);
			p2.setX(fx-4);
			p2.setY(fy-2);
			p3.setX(fx-4);
			p3.setY(fy+2);
			p4.setX(fx);
			p4.setY(fy+2);
		}
		else{
			p1.setX(fx);
			p1.setY(fy-2);
			p2.setX(fx+4);
			p2.setY(fy-2);
			p3.setX(fx+4);
			p3.setY(fy+2);
			p4.setX(fx);
			p4.setY(fy+2);
		}
	
	if(t1.y() == t2.y())
		if(t1.y() == to->y()){
			p5.setX(tx-2);
			p5.setY(ty);
			p6.setX(tx-2);
			p6.setY(ty-4);
			p7.setX(tx+2);
			p7.setY(ty-4);
			p8.setX(tx+2);
			p8.setY(ty);
		}
		else{
			p5.setX(tx-2);
			p5.setY(ty);
			p6.setX(tx-2);
			p6.setY(ty+4);
			p7.setX(tx+2);
			p7.setY(ty+4);
			p8.setX(tx+2);
			p8.setY(ty);
		}
	else
		if(t1.x() == to->x()){
			p5.setX(tx);
			p5.setY(ty-2);
			p6.setX(tx-4);
			p6.setY(ty-2);
			p7.setX(tx-4);
			p7.setY(ty+2);
			p8.setX(tx);
			p8.setY(ty+2);
		}
		else{
			p5.setX(tx);
			p5.setY(ty-2);
			p6.setX(tx+4);
			p6.setY(ty-2);
			p7.setX(tx+4);
			p7.setY(ty+2);
			p8.setX(tx);
			p8.setY(ty+2);
		}
	QPointArray pb(4);
	
	QPen pen(fldcolors(k));
	QBrush brush(fldcolors(k));
	p.setPen(pen);
	p.setBrush(brush);
	pb.putPoints(0, 4, p1.x(), p1.y(), p2.x(), p2.y(), p3.x(), p3.y(), p4.x(), p4.y());
	p.drawPolygon(pb);
	
	pen.setColor(fldcolors(i));
	brush.setColor(fldcolors(i));
	p.setPen(pen);
	p.setBrush(brush);
	pb.putPoints(0, 4, p5.x(), p5.y(), p6.x(), p6.y(), p7.x(), p7.y(), p8.x(), p8.y());
	p.drawPolygon(pb);
	
	pen.setColor("red");
	brush.setColor("red");
	p.setPen(pen);
	p.setBrush(brush);
	if(iswound(p2, p6, p7, p3)){
		pb.putPoints(0, 4, p2.x(), p2.y(), p7.x(), p7.y(), p6.x(), p6.y(), p3.x(), p3.y());
		pa.putPoints(0, 8, p1.x(), p1.y(), p2.x(), p2.y(), p7.x(),
	  									 p7.y(), p5.x(), p5.y(), p8.x(), p8.y(),
											 p6.x(), p6.y(), p3.x(), p3.y(), p4.x(),
											 p4.y());
	}
	else{
		pb.putPoints(0, 4, p2.x(), p2.y(), p6.x(), p6.y(), p7.x(), p7.y(), p3.x(), p3.y());
		pa.putPoints(0, 8, p1.x(), p1.y(), p2.x(), p2.y(), p6.x(),
											 p6.y(), p5.x(), p5.y(), p8.x(), p8.y(),
											 p7.x(), p7.y(), p3.x(), p3.y(), p4.x(),
											 p4.y());
	}
	p.drawPolygon(pb);
	setPoints(pa);
}

