/***************************************************************************
                          ctable.cpp  -  description
                             -------------------
    begin                : Wed May 2 2001
    copyright            : (C) 2001 by Mihnea Balta
    email                : mihnea@tomrad.ro
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ctable.h"
#include <iostream>

static uint nexttblid = 0;
static uint nextfldid = 0;
static uint nextrelid = 10000;

CField::CField(uint fid=0){
	if(fid)
		id = fid;
	else
		id = ++nextfldid;
	refcnt = 0;
	flags = 0;
	refcnt = 0;
	size = 0;
//	cerr << "Creating CField with id " << id << endl;
}

CField::~CField(){
//	cerr << "Deleting CField with id " << id << endl;
}

CRelation::CRelation(uint rid=0){
	if(rid)
		id = rid;
	else
		id = ++nextrelid;
	fromFld = 0;
	toFld = 0;
//	cerr << "Creating CRelation with id " << id << endl;
}

CRelation::~CRelation(){
//	cerr << "Deleting CRelation with id " << id << endl;
}

CTable::CTable(uint cid=0){
	if(cid)
		id = cid;
	else
		id = ++nexttblid;
	fields.clear();
	relations.clear();
//	cerr << "Creating CTable with id " << id << endl;
}

CTable::~CTable(){
//	cerr << "Deleting CTable with id " << id << endl;
}

void CTable::insertField(CField *newfld){
	newfld->parent = id;
	fields.push_back(*newfld);
}

void CTable::delField(uint fldid){
	for(fldlist::iterator it = fields.begin(); it!=fields.end(); it++)
		if( it->getId() == fldid){
			fields.erase(it, it);
			break;
		}
}

/****
	It is safe (and easier) to pass a CRelation* parameter
	to this function; however, the pointer is
	not inserted anywhere, just the id contained
	in it. If absolutely necessary i will overload
	this with insertRelation(uid *relid);
****/
CField* CTable::getFldfromId(int f_id){
	for(fldlist::iterator it = fields.begin(); it!=fields.end(); it++){
		if(it->getId() == f_id)
			return &(*it);
	}
	//we should never get here
	return 0;
}

void CTable::insertRelation(CRelation *rel){
	relations.push_back(rel->getId());
	CField *f;
	if(rel->fromTbl == id){
		f = getFldfromId(rel->fromFld);
		f->flags |= FIELD_FK;
	}
	else{
		f = getFldfromId(rel->toFld);
		f->refcnt++;
	}
}

void CTable::delRelation(uint relid, uint fldid){
	CField *fld = getFldfromId(fldid);
	if(fld->flags & FIELD_FK)
		fld->flags &= !FIELD_FK;
	else
		fld->refcnt--;
	for(idlist::iterator it = relations.begin(); it!=relations.end(); it++)
		if(*it == relid){
			relations.erase(it, it);
			break;
		}
}

/*****
	This function should never be called to replace
	a set of relations with another. It is only intended
	for relation list intialization, when copying a table
*****/
void CTable::setRelations(idlist &newrel){
	relations = newrel;
}

CField* CTable::getFldfromName(QString name){
	for(fldlist::iterator it = fields.begin(); it!=fields.end(); it++){
		if(it->name == name)
			return &(*it);
	}
	return 0;
}
