/***************************************************************************
                          ctable.h  -  description
                             -------------------
    begin                : Wed May 2 2001
    copyright            : (C) 2001 by Mihnea Balta
    email                : mihnea@tomrad.ro
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CTABLE_H
#define CTABLE_H

#include <qstring.h>
#include <list>

/****
  @author Mihnea Balta
  CTable contains the functional info about a table in the project.
  It is contained by CTableItem.
****/

#define FIELD_PK			0x00000001
#define FIELD_UNIQUE	0x00000002
#define FIELD_NOTNULL	0x00000004
#define FIELD_FK			0x00000008

/****	PLUGIN NOTE
	This table is exported from the plugin module
****/
static const char* const FldTypes[] = {
	"int",
	"text",
	"date",
	"varchar",
	"char",
	"smallint",
	"constraint",
	"check",
	0
};

/****
ValidRef[i] is:
	1 if fields of type FldTypes[i] can be used as a PK/FK.
	0 otherwise
****/
/****	PLUGIN NOTE
	This table is exported from the plugin module
****/
static const int ValidRef[] = {
	1, 1, 1, 1, 1, 1, 0, 0, 0
};

typedef unsigned int uint;

class CTable;

class CField{
private:
	uint		id;
public:
	CField(uint);
	~CField();
	uint		getId() { return id; }
	QString	name;    	//field name
	int			type_idx;	//index in FldTypes[] type table
	int			flags;		//unique, not null, PK, etc.
	int			refcnt;		//reference count
	int			size;			//data type size
	QString	defval;		//default value. freeform
	QString	comment;	//no comment ;)
	uint		parent;		//the table to which the field belongs
};

struct CRelation{
private:
	uint		id;
public:
	CRelation(uint);
	~CRelation();
	uint		getId() { return id; }
	QString	name;
	QString	comment;
	uint		toTbl;
	uint		toFld;
	uint		fromTbl;
	uint		fromFld;
};

typedef list<CField> fldlist;
typedef list<uint> idlist;

class CTable {
private:
	QString	tname;
	fldlist	fields;
	idlist	relations;
	uint		id;
public:
	CTable(uint);
	~CTable();
	uint				getId() { return id; }
	void 				setName(const char *name) { tname = name; }
	const char	*name() const { return tname.latin1(); }
	//fields
	fldlist			getFields() const { return fields; }
	void				insertField(CField *newfld);
	void				delField(uint fldid);
	void				clearFields() { fields.clear(); }
	//relations
	idlist			getRelations() const { return relations;}
	void				setRelations(idlist &newrel);
	void				insertRelation(CRelation *rel);
	void				delRelation(uint relid, uint fldid);
	void				clearRelations() { relations.clear(); }
	CField			*getFldfromId(int);
	CField			*getFldfromName(QString name);
};

#endif
