/***************************************************************************
                          ctableed.cpp  -  description
                             -------------------
    begin                : Wed May 2 2001
    copyright            : (C) 2001 by Mihnea Balta
    email                : mihnea@tomrad.ro
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <math.h>
#include "ctableed.h"
#include "cview.h"
#include "plugin.h"

extern CPlugin crp;

CTableEd::CTableEd(bool newT, QWidget *parent, const char *name ) : CmetaTableEd(parent, name, TRUE) {
	fldTable->setSelectionMode(QTable::Single);
	if(newT)
		setCaption("New table");
	fldTable->setNumCols(9+crp.modifCount());
	fldTable->horizontalHeader()->setLabel( 0, "Name");
	fldTable->horizontalHeader()->setLabel( 1, "Type");
	fldTable->horizontalHeader()->setLabel( 2, "Size");
	fldTable->horizontalHeader()->setLabel( 3, "Key");
	fldTable->horizontalHeader()->setLabel( 4, "Not null");
	fldTable->horizontalHeader()->setLabel( 5, "Unique");
	fldTable->horizontalHeader()->setLabel( 6, "Default/Value");
	fldTable->horizontalHeader()->setLabel( 7, "Comment");
	fldTable->horizontalHeader()->setLabel( 8, "Reference count");
	for(int i=0; i<crp.modifCount(); i++)
		fldTable->horizontalHeader()->setLabel(9+i, crp.getModifName(i));
	v = new CAlphaValidator;
	tblname->setValidator(v);
	t_id = 0;
}

CTableEd::~CTableEd(){
	delete v;
}

void CTableEd::fld_new(){
	int i;
	CFldEdit fe(this);
	
	if(!fe.exec())
		return;
	QString nm = fe.nameEd->text();
	if(nm.length() == 0){
		KMessageBox::sorry(this, "You didn't specify a name for the field", "Error");
		return;
	}
	for(i=0; i<fldTable->numRows(); i++)
		if( (fldTable->text(i, 0).length() == nm.length()) &&
				(nm.contains(fldTable->text(i, 0), false)>0) ){
			KMessageBox::sorry(this, "Cannot create 2 fields with the same name", "Error");
			return;
		}
	fldTable->setNumRows(fldTable->numRows()+1);
	QTableItem *it = new QTableItem(fldTable, QTableItem::Never, nm);
	fldTable->setItem(fldTable->numRows()-1, 0, it);
	it = new QTableItem(fldTable, QTableItem::Never, fe.typeCombo->currentText());
	fldTable->setItem(fldTable->numRows()-1, 1, it);
	it = new QTableItem(fldTable, QTableItem::Never, fe.sizeEdit->text());
	fldTable->setItem(fldTable->numRows()-1, 2, it);
	it = new QTableItem(fldTable, QTableItem::Never, fe.keyCombo->currentText());
	fldTable->setItem(fldTable->numRows()-1, 3, it);
	if(fe.notNullCheck->isChecked())
		it = new QTableItem(fldTable, QTableItem::Never, QString("Yes"));
	else
		it = new QTableItem(fldTable, QTableItem::Never, QString("No"));
	fldTable->setItem(fldTable->numRows()-1, 4, it);
	if(fe.uniqueCheck->isChecked())
		it = new QTableItem(fldTable, QTableItem::Never, QString("Yes"));
	else
		it = new QTableItem(fldTable, QTableItem::Never, QString("No"));
	fldTable->setItem(fldTable->numRows()-1, 5, it);
	it = new QTableItem(fldTable, QTableItem::Never, fe.valEdit->text());
	fldTable->setItem(fldTable->numRows()-1, 6, it);
	it = new QTableItem(fldTable, QTableItem::Never, fe.commentEdit->text());
	fldTable->setItem(fldTable->numRows()-1, 7, it);
	it = new QTableItem(fldTable, QTableItem::Never, "0");
	fldTable->setItem(fldTable->numRows()-1, 8, it);
	for(int i=0; i<crp.modifCount(); i++){
		if(fe.modifs[i+2]->isChecked())
			it = new QTableItem(fldTable, QTableItem::Never, "Yes");
		else
			it = new QTableItem(fldTable, QTableItem::Never, "No");
		fldTable->setItem(fldTable->numRows()-1, 9+i, it);
	}
	fieldIDs.push_back(0);
}

//QComboBox really should provide this...
void CTableEd::setcombo(QComboBox *qb, const QString &s){
	int i;
	for(i=0; i<qb->count(); i++)
		if(s == qb->text(i)){
			qb->setCurrentItem(i);
			return;
		}
	//we'd better never get here
	qb->setCurrentItem(0);
}

void CTableEd::setcheck(QCheckBox *qc, const QString &s){
	if(s == "Yes")
		qc->setChecked(true);
	else
		qc->setChecked(false);
}

void CTableEd::fld_edit(){
	int cr = fldTable->currentRow();
	
	int i;
	CFldEdit fe(this);
	
	fe.nameEd->setText(fldTable->text(cr, 0));
	setcombo(fe.typeCombo, fldTable->text(cr, 1));
	fe.type_chg();
	fe.sizeEdit->setText(fldTable->text(cr, 2));
	if(fldTable->text(cr, 3) == "Foreign"){
		fe.keyCombo->insertItem("Foreign");
		fe.keyCombo->setEnabled(false);
		fe.notNullCheck->setEnabled(false);
		fe.typeCombo->setEnabled(false);
		fe.sizeEdit->setEnabled(false);
	}
	setcombo(fe.keyCombo, fldTable->text(cr, 3));
	fe.key_type_chg();
	setcheck(fe.notNullCheck, fldTable->text(cr, 4));
	setcheck(fe.uniqueCheck, fldTable->text(cr, 5));
	fe.valEdit->setText(fldTable->text(cr, 6));
	fe.commentEdit->setText(fldTable->text(cr, 7));
	for(int i=0; i<crp.modifCount(); i++)
		if(fldTable->text(cr, i+9) == "Yes")
			fe.modifs[i+2]->setChecked(true);
		else
			fe.modifs[i+2]->setChecked(false);
	
	if(!fe.exec())
		return;
	
	QString nm = fe.nameEd->text();
	if(nm.length() == 0){
		KMessageBox::sorry(this, "You didn't specify a name for the field", "Error");
		return;
	}
	for(i=0; i<fldTable->numRows(); i++)
		if( (fldTable->text(i, 0).length() == nm.length()) &&
				(nm.contains(fldTable->text(i, 0), false)>0) && (cr != i) ){
			KMessageBox::sorry(this, "Cannot create 2 fields with the same name", "Error");
			return;
		}
	fldTable->setText(cr, 0, nm);
	fldTable->setText(cr, 1, fe.typeCombo->currentText());
	fldTable->setText(cr, 2, fe.sizeEdit->text());
	fldTable->setText(cr, 3, fe.keyCombo->currentText());
	if(fe.notNullCheck->isChecked())
		fldTable->setText(cr, 4, "Yes");
	else
		fldTable->setText(cr, 4, "No");
	if(fe.uniqueCheck->isChecked())
		fldTable->setText(cr, 5, "Yes");
	else
		fldTable->setText(cr, 5, "No");
	fldTable->setText(cr, 6, fe.valEdit->text());
	fldTable->setText(cr, 7, fe.commentEdit->text());
	for(int i=0; i<crp.modifCount(); i++)
		if(fe.modifs[i+2]->isChecked())
			fldTable->setText(cr, 9+i, "Yes");
		else
			fldTable->setText(cr, 9+i, "No");
}

void CTableEd::fld_del(){
	int nr = fldTable->numRows();
  if(nr < 1)
  	return;
  if(fldTable->text(fldTable->currentRow(), 3) == "Foreign"){
  	KMessageBox::sorry(this, "You can't delete foreign keys. Delete the relation first.", "Error");
  	return;
  }
	int i=0;
  idlist::iterator it = fieldIDs.begin();
  int cr = fldTable->currentRow();
  for(;i<cr; i++, it++);
	//it's safe to use the old table's rellist because
	//the user can't add referenced keys
/*
	CView *par = (CView*)parent();
	if(!relationlist.empty())
		for(idlist::iterator rit = relationlist.begin(); rit != relationlist.end(); rit++)
			if(par->getRelfromId(*rit)->toFld == *it){
		  	KMessageBox::sorry(this, "Can't delete: this field is referenced by another table's Foreign Key.", "Error");
 		 		return;
			}
*/
	if(fldTable->text(cr, 8) != "0"){
  	KMessageBox::sorry(this, "Can't delete: this field is referenced by another table's Foreign Key.", "Error");
 		return;
 }
  fieldIDs.erase(it, it);
	for(; i<nr-1; i++)
		for(int j=0; j<fldTable->numCols(); j++)
			fldTable->setText(i, j, fldTable->text(i+1, j));
	fldTable->setNumRows(nr-1);
}

CTable* CTableEd::getTable(){

	CTable *table = new CTable(t_id);
	table->setRelations(relationlist);
	
	table->setName(tblname->text());
	idlist::iterator it = fieldIDs.begin();
	for(int i=0; i<fldTable->numRows(); i++, it++){
		CField	*fld = new CField(*it);
		fld->name = fldTable->text(i, 0);
		int j=0;
		for(;j<crp.typeCount(); j++)
			if( fldTable->text(i, 1) == crp.typeName(j) )
				break;
		/*
		while(FldTypes[j] && (fldTable->text(i, 1)!=FldTypes[j]) )
			j++;
		*/
		fld->type_idx = j;
		fld->size = abs(fldTable->text(i, 2).toInt());
		fld->flags = 0;
		if(fldTable->text(i, 3) == "Primary")
			fld->flags|=FIELD_PK;
		if(fldTable->text(i, 3) == "Foreign")
			fld->flags|=FIELD_FK;
		if(fldTable->text(i, 4) == "Yes")
			fld->flags|=FIELD_NOTNULL;
		if(fldTable->text(i, 5) == "Yes")
			fld->flags|=FIELD_UNIQUE;
		for(int j=0; j<crp.modifCount(); j++)
			if(fldTable->text(i, j+9) == "Yes")
				fld->flags|=crp.getModifMask(j);
		fld->defval = fldTable->text(i, 6);
		fld->comment = fldTable->text(i, 7);
		QString s = fldTable->text(i, 8);
		fld->refcnt = s.toInt();
		table->insertField(fld);
	}
	return table;
}

void CTableEd::setTable(CTable *newtbl){
	tblname->setText(newtbl->name());
	t_id = newtbl->getId();
	fieldIDs.clear();
	
	fldlist l = newtbl->getFields();
	relationlist = newtbl->getRelations();
	int i=0;
	for(fldlist::iterator it=l.begin(); it!=l.end(); it++, i++){
		fldTable->setNumRows(fldTable->numRows()+1);
		fieldIDs.push_back(it->getId());
		QTableItem *fld = new QTableItem(fldTable, QTableItem::Never, it->name);
		fldTable->setItem(i, 0, fld);
		QString s;
		fld = new QTableItem(fldTable, QTableItem::Never, QString(crp.typeName(it->type_idx)));
		fldTable->setItem(i, 1, fld);
		s.sprintf("%d", it->size);
		fld = new QTableItem(fldTable, QTableItem::Never, s);
		fldTable->setItem(i, 2, fld);
		if(it->flags & FIELD_PK)
			fld = new QTableItem(fldTable, QTableItem::Never, "Primary");
		else
			if(it->flags & FIELD_FK)
				fld = new QTableItem(fldTable, QTableItem::Never, "Foreign");
			else
				fld = new QTableItem(fldTable, QTableItem::Never, "No");
		fldTable->setItem(i, 3, fld);
		if(it->flags & FIELD_NOTNULL)
			fld = new QTableItem(fldTable, QTableItem::Never, "Yes");
		else
			fld = new QTableItem(fldTable, QTableItem::Never, "No");
		fldTable->setItem(i, 4, fld);
		if(it->flags & FIELD_UNIQUE)
			fld = new QTableItem(fldTable, QTableItem::Never, "Yes");
		else
			fld = new QTableItem(fldTable, QTableItem::Never, "No");
		fldTable->setItem(i, 5, fld);
		for(int j=0; j<crp.modifCount(); j++){
			if(it->flags & crp.getModifMask(j))
				fld = new QTableItem(fldTable, QTableItem::Never, "Yes");
			else
				fld = new QTableItem(fldTable, QTableItem::Never, "No");
			fldTable->setItem(i, 9+j, fld);
		}
		fld = new QTableItem(fldTable, QTableItem::Never, it->defval);
		fldTable->setItem(i, 6, fld);
		fld = new QTableItem(fldTable, QTableItem::Never, it->comment);
		fldTable->setItem(i, 7, fld);
		s.sprintf("%d", it->refcnt);
		fld = new QTableItem(fldTable, QTableItem::Never, s);
		fldTable->setItem(i, 8, fld);
	}
}
