/***************************************************************************
                          ctableitem.cpp  -  description
                             ----------------------
    begin                : Sun Apr 29 2001
    copyright            : (C) 2001 by Mihnea Balta
    email                : mihnea@tomrad.ro
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ctableitem.h"
#include "fldcolors.h"

CTableItem::CTableItem(CTable *table, int x, int y, QCanvas *canvas) : QCanvasRectangle(x, y, 0, 0, canvas){
	hasFocus = false;
	tbl = table;
	visible = true;
	resize();
}

CTableItem::~CTableItem(){
	delete tbl;
}

void CTableItem::resize(){
	QFont f;
	f.setBold(true);
	f.setPointSize(12);
	int max = QFontMetrics(f).width(tbl->name()) + 10;
	int h = QFontMetrics(f).height() + 7;
	f.setBold(false);
	f.setPointSize(10);
	fldlist l = tbl->getFields();
	int k=0, i=1;
	for(fldlist::iterator it = l.begin(); it!=l.end(); it++){
		CField item = (CField)(*it);
		if(ValidRef[item.type_idx]){
			i++;
			k = QFontMetrics(f).width(item.name)+30+16;
			if(k>max)
				max = k;
		}
	}
	h += i*(QFontMetrics(f).height()+2);
	setSize(max, h);
}

void CTableItem::drawShape(QPainter &p){
	if(!visible)
		return;
	// draw bounding rectangle
	QBrush brush(QColor(120, 120, 120));
	if(hasFocus)
		brush.setColor(QColor(200, 200, 200));
	p.setBrush(brush);
	p.drawRect(x(), y(), width(), height());
	// draw the table name
	QFont f;
	f.setBold(true);
	f.setPointSize(12);
	p.setFont(f);
	
	int tw = QFontMetrics(p.font()).width(tbl->name());
	int th = QFontMetrics(p.font()).height();
	int ofs = (width()-tw)/2;
	brush.setColor(QColor(255, 255, 180));
	p.setBrush(brush);
	p.drawRect(x()+ofs-2, y()+2, tw+4, th+3);
	p.drawText(x()+ofs, y()+3, tw, th, AlignCenter, tbl->name());
	f.setBold(false);
	f.setPointSize(10);
	p.setFont(f);
	QPixmap pk_pix((const char **)pk);
	QPixmap fk_pix((const char **)fk);
	int i=(int)y()+th+7;
	th = QFontMetrics(p.font()).height();
	fldlist l = tbl->getFields();
	int k = 0;
	for(fldlist::iterator it = l.begin(); it!=l.end(); it++){
		CField item = (CField)(*it);
		if(ValidRef[item.type_idx]){
			i+=th;
			brush.setColor(QColor(255, 255, 180));
			p.setBrush(brush);
			p.drawText(x()+20, i, item.name);
			if(item.flags & FIELD_PK)
				p.drawPixmap(x()+2, i-th+1, pk_pix);
			if(item.flags & FIELD_FK){
				brush.setColor(fldcolors(k));
				p.setBrush(brush);
				p.drawRect(QRect(x()+width()-18, i-th+5, 8, 8));
				p.drawPixmap(x()+2, i-th+1, fk_pix);
				k++;
			}
			if(item.refcnt > 0){
				brush.setColor(fldcolors(k));
				p.setBrush(brush);
				p.drawEllipse(QRect(x()+width()-18, i-th+5, 8, 8));
				k++;
			}
		}
	}
}

bool CTableItem::hit(const QPoint &p) {
	int ix = int(x());
	int iy = int(y());
	int px = int(p.x());
	int py = int(p.y());
	int w = int(width());
	int h = int(height());
	
	if( (px>=ix) && (py>=iy) &&
			(px<=ix+w) && (py<=iy+h) ){
		distUL.setX(px-ix);
		distUL.setY(py-iy);
		distLR.setX(w+ix-px);
		distLR.setY(h+iy-py);
		return TRUE;
	}
	else
		return FALSE;
}

CTable* CTableItem::getTable(){
	return tbl;
}

void CTableItem::setFocus(bool focus){
	hasFocus = focus;
	this->canvas()->setAllChanged();
	this->canvas()->update();
}

bool CTableItem::focused(){
	return hasFocus;
}

void CTableItem::setTable(CTable *newtable){
	delete tbl;
	tbl = newtable;
	resize();
}
