/***************************************************************************
                          cview.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 2001
    copyright            : (C) 2001 by Mihnea Balta
    email                : mihnea@tomrad.ro
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dbdesign.h"
#include "cview.h"
#include <iostream>
#include <kfiledialog.h>
#include <qtextstream.h>
#include <qxml.h>
#include <vector>
#include <hash_map>
#include <string.h>
#include "plugin.h"
#include "svrdlg.h"
#include "seqedit.h"
#include <knuminput.h>
#include <qmap.h>


extern CPlugin crp;

CView::CView(QWidget* parent,	const char* name) : QCanvasView(0, parent, name, 0)
{
	cnv = new QCanvas(1000, 1000);
	cnv->setDoubleBuffering(TRUE);
	setCanvas(cnv);
	insertChild(cnv);
	moving = 0;
	focusItem = 0;
	focusRel = 0;
	focusSeq = 0;
	ctxMenu = new QPopupMenu(this);
	menu_id[0] = ctxMenu->insertItem("");
	ctxMenu->insertSeparator();
	menu_id[1] = ctxMenu->insertItem("New &relation", this, SLOT(slotNewRel()));
	menu_id[2] = ctxMenu->insertItem("&New table", this, SLOT(slotNewTbl()));
	menu_id[3] = ctxMenu->insertItem("&Edit table", this, SLOT(slotEditTbl()));
	menu_id[4] = ctxMenu->insertItem("&Delete table", this, SLOT(slotDelTbl()));
	menu_id[5] = ctxMenu->insertItem("E&xport to...", this, SLOT(slotExportTbl()));
	nrtbl = 0;
	modified = false;
	sequences.clear();
	view_map all;
	all.clear();
	views["All"] = all;
	crview = "All";
}

CView::~CView(){
}

void CView::contentsMouseMoveEvent(QMouseEvent* e){
	if ( moving ){
		int	ix = e->pos().x();
		int iy = e->pos().y();
		if(ix - moving->distUL.x() < 0)
			ix = moving->distUL.x();
		if(ix + moving->distLR.x() > cnv->width())
			ix = cnv->width() - moving->distLR.x();
		if(iy - moving->distUL.y() < 0)
			iy = moving->distUL.y();
		if(iy + moving->distLR.y() > cnv->height())
			iy = cnv->height() - moving->distLR.y();
		if(QPoint(ix - moving_start.x(), iy - moving_start.y()).manhattanLength() < QApplication::startDragDistance() )
			return;
		moving->moveBy(ix - moving_start.x(),
									 iy - moving_start.y());
		moving_start.setX(ix);
		moving_start.setY(iy);
		modified = true;
		//TEST THIS!
		idlist l = moving->getTable()->getRelations();
		for(idlist::iterator it = l.begin(); it != l.end(); it++){
			//get the relations to redraw themselves; since there's no
			//invalidate method for QCanvasItem, this is the only hack
			//i could think of
			getRelItemfromId(*it)->hide();
			getRelItemfromId(*it)->show();
		}
		cnv->update();
  }
}

void CView::contentsMouseDoubleClickEvent(QMouseEvent* e){
	QCanvasItemList l=cnv->collisions(e->pos());
	CTableItem *item = NULL;
	for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it){
		if ( (*it)->rtti() == tableRTTI ){
			CTableItem *item= (CTableItem*)(*it);
	    if ( !item->hit( e->pos() ) )
				continue;
		}
		item= (CTableItem*)(*it);
		if(focusItem!=item){
			if(focusItem)
				focusItem->setFocus(false);
			focusItem = item;
			focusItem->setFocus(true);
		}
		else
			focusItem->setFocus(true);
		break;
	}
	slotEditTbl();
}

void CView::contentsMousePressEvent(QMouseEvent* e){
	QCanvasItemList l=cnv->collisions(e->pos());
	CTableItem *item = 0;
	bool found = false;
	
	//check for collision
	for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it)
		if ( (*it)->rtti() == tableRTTI ){
			item = (CTableItem*)(*it);
	    if ( item->hit( e->pos() ) ){
	    	found = true;
				break;
			}
		}
	
	//if no collision, unset moving, loose focus and display New Table menu if RMB
	if(!found){
		moving = 0;
		if(focusItem)
			focusItem->setFocus(false);
		if(e->button() == RightButton){
			ctxMenu->changeItem(menu_id[0], "-= (empty) =-");
			for(int i=3; i<6; i++)
				ctxMenu->setItemEnabled(menu_id[i], false);
			QPoint p = mapToGlobal(e->pos());
			p.setX(p.x()+2);
			ctxMenu->popup(p);
			newTableAt = e->pos();
		}
		return;
	}
	
	//hit; change focus if necessary
	if(item != focusItem){
		if(focusItem)
			focusItem->setFocus(false);
		item->setFocus(true);
		focusItem = item;
	}
	else
		focusItem->setFocus(true);
		
	//if LMB, set moving
	if(e->button() == LeftButton){
		moving = item;
		moving_start = e->pos();
		return;
	}
	
	//if RMB, display full popup
	if(e->button() == RightButton){
		QString s;
		s.sprintf("-= %s =-", item->getTable()->name());
		ctxMenu->changeItem(menu_id[0], s);
		for(int i=3; i<6; i++)
			ctxMenu->setItemEnabled(menu_id[i], true);
		QPoint p = mapToGlobal(e->pos());
		p.setX(p.x()+2);
		ctxMenu->popup(p);
		newTableAt = e->pos();
	}
}

void CView::slotNewTbl(){
	CTableEd ted(true, this);
	bool ok = false;
	
	if(nrtbl == MAXTABLES){
		KMessageBox::sorry(this, "Sorry, you cannot create more tables", "Error");
		return;
	}
	
	while(!ok){
		if (!ted.exec())
			return;
		CTable *table = ted.getTable();
		QString s(table->name());
		ok = true;
		if(s.length() == 0){
			KMessageBox::sorry(this, "You didn't specify a name for the table", "Error");
			ok = false;
		}
		else{
			if(!tblNameUniq(s)){
				KMessageBox::sorry(this, "Duplicate table name", "Error");
				ok = false;
			}
			if( ok && (ted.fldTable->numRows() == 0) ){
				KMessageBox::sorry(this, "Empty table", "Error");
				ok = false;
			}
		}
		if(!ok)
			delete table;
	}
	CTable *table = ted.getTable();
	views["All"][table->getId()] = true;
	if(crview != "All")
		views[crview][table->getId()] = true;
	nrtbl++;
	CTableItem *t = new CTableItem(table, newTableAt.x(), newTableAt.y(), cnv);
	t->setZ(table->getId());
	t->show();
	cnv->update();
	
	DbDesign *prn = (DbDesign *)(parent());
	prn->ltree->insTableLast((const char *)(table->name()), t);
	
	modified = true;
}

void CView::slotEditTbl(){
	//DO NOT COMPILE WITH COMPLETE BOOLEAN EVAL!
	if( (!focusItem) || (!focusItem->focused()) ){
		KMessageBox::sorry(this, "Nothing selected!", "Error");
		return;
	}
	CTableEd ted(false, this);
	bool ok = false;
	ted.setTable(focusItem->getTable());
	while(!ok){
		if (!ted.exec())
			return;
		CTable *table = ted.getTable();
		QString s(table->name());
		ok = true;
		if(s.length() == 0){
			KMessageBox::sorry(this, "You didn't specify a name for the table", "Error");
			ok = false;
		}
		else
			if(s != focusItem->getTable()->name()){
				if(!tblNameUniq(s)){
					KMessageBox::sorry(this, "Duplicate table name", "Error");
					ok = false;
				}
				if( ok && (ted.fldTable->numRows() == 0) ){
					KMessageBox::sorry(this, "Empty table", "Error");
					ok = false;
				}
			}
		if(!ok)
			delete table;
	}
	//update table
	CTable *table = ted.getTable();
	DbDesign *main = (DbDesign *)(parent());
	main->ltree->chgTableItem((const char *)(focusItem->getTable()->name()), (const char *)(table->name()));
	focusItem->setTable(table);
	modified = true;
}

void CView::slotDelTbl(){
	//DO NOT COMPILE WITH COMPLETE BOOLEAN EVAL!
	if( (!focusItem) || (!focusItem->focused()) ){
		KMessageBox::sorry(this, "Nothing selected!", "Error");
		return;
	}
	if(!focusItem->getTable()->getRelations().empty()){
		KMessageBox::sorry(this, "Cannot delete: This table has relations.", "Error");
		return;
	}
	
	nrtbl--;
	for(viewlist::Iterator it=views.begin(); it!=views.end(); it++){
		view_map items = views[it.key()];
		items.remove(focusItem->getTable()->getId());
	}
	DbDesign *main = (DbDesign *)(parent());
	main->ltree->delTableItem((const char *)(focusItem->getTable()->name()));
	delete focusItem;
	focusItem = 0;
	cnv->update();
	modified = true;
}

void CView::slotExportTbl(){
	QString *sql;
	QFile f;
	
	for(;;){
		QString fnm = KFileDialog::getSaveFileName(QString("::exporttable"), QString("*.sql|SQL scripts\n*|All files"), (QWidget *)parent(), QString("Export table..."));
		if (fnm.isNull())
			return;
		if(fnm.right(4) != ".sql")
			fnm += QString(".sql");
		f.setName(fnm);
		if(!f.exists())
			break;
		else
			if (KMessageBox::questionYesNo(this, "The file exists. Overwrite?") == KMessageBox::Yes)
				break;
	}
	sql = getTblSql(focusItem->getTable());
	if(f.open(IO_WriteOnly)){
		QTextStream t(&f);
		t << *sql;
		f.close();
	}
	else
		KMessageBox::error(this, "Could not write to file.");
}

void CView::setFocusItem(CTableItem *item){
	if(item!=focusItem){
		if(focusItem)
			focusItem->setFocus(false);
		item->setFocus(true);
		focusItem = item;
	}
	else
		focusItem->setFocus(true);
	ensureVisible(item->x(), item->y());
}

void CView::setFocusRel(CRelationItem *item){
	focusRel = item;
}

void CView::setFocusSeq(CSequence *item){
	focusSeq = item;
}

CTable* CView::getTblfromId(uint cid){
	QCanvasItemList l = cnv->allItems();
	for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it)
		if ( (*it)->rtti() == tableRTTI ){
			CTableItem *item = (CTableItem*)(*it);
	    if ( item->getTable()->getId() == cid )
	    	return item->getTable();
		}
	//we should never get here...
	qWarning("CView::getTblfromId() : reached end of function, returning 0; the app will most likely crash.");
	return 0;
}

CRelation* CView::getRelfromId(uint rid){
	QCanvasItemList l = cnv->allItems();
	for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it)
		if ( (*it)->rtti() == relationRTTI ){
			CRelationItem *item = (CRelationItem*)(*it);
	    if ( item->getRel()->getId() == rid )
	    	return item->getRel();
		}
	// ... or here, for that matter.
	qWarning("CView::getRelfromId() : reached end of function, returning 0; the app will most likely crash.");
	return 0;
}

CTableItem* CView::getTblItemfromId(uint cid){
	QCanvasItemList l = cnv->allItems();
	for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it)
		if ( (*it)->rtti() == tableRTTI ){
			CTableItem *item = (CTableItem*)(*it);
	    if ( item->getTable()->getId() == cid )
	    	return item;
		}
	qWarning("CView::getTblItemfromId() : reached end of function, returning 0; the app will most likely crash.");
	return 0;
}

CRelationItem* CView::getRelItemfromId(uint rid){
	QCanvasItemList l = cnv->allItems();
	for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it)
		if ( (*it)->rtti() == relationRTTI ){
			CRelationItem *item = (CRelationItem*)(*it);
	    if ( item->getRel()->getId() == rid )
	    	return item;
		}
	qWarning("CView::getRelItemfromId() : reached end of function, returning 0; the app will most likely crash.");
	return 0;
}

bool CView::relNameUniq(QString &s){
	QCanvasItemList l = cnv->allItems();
	for(QCanvasItemList::Iterator it = l.begin(); it!=l.end(); it++)
		if((*it)->rtti() == relationRTTI){
			CRelationItem *item = (CRelationItem*)(*it);
			if(s == item->getRel()->name)
				return false;
		}
	return true;
}

bool CView::tblNameUniq(QString &s){
	QCanvasItemList l = cnv->allItems();
	for(QCanvasItemList::Iterator it = l.begin(); it!=l.end(); it++)
		if((*it)->rtti() == tableRTTI){
			CTableItem *item = (CTableItem*)(*it);
			if(s == item->getTable()->name())
				return false;
		}
	return true;
}

void CView::slotNewRel(){
	if(cnv->allItems().count() < 2){
		KMessageBox::sorry(this, "You can't create a relation with less than 2 tables!", "Error");
		return;
	}
	CRelEdit	red(this);
	bool ok = false;
	
	while(!ok){
		if(!red.exec())
			return;
		ok = false;
		QString s = red.relName->text();
		if(!relNameUniq(s)){
			KMessageBox::sorry(this, "Cannot create 2 relations with the same name", "Error");
			continue;
		}
		ok = true;
	}
	CRelation *rel = red.getRelation();
	views["All"][rel->getId()] = true;
	if(crview != "All")
		views[crview][rel->getId()] = true;
	getTblfromId(rel->fromTbl)->insertRelation(rel);
	getTblfromId(rel->toTbl)->insertRelation(rel);
	CRelationItem *ri = new CRelationItem(rel, cnv);
	ri->setZ(rel->getId());
	ri->show();
	cnv->setAllChanged();
	cnv->update();
	DbDesign *prn = (DbDesign *)(parent());
	prn->ltree->insRelLast(rel->name, ri);
	modified = true;
}

void CView::slotEditRel(){
	CRelEdit red(this);
	bool ok = false;
	
	red.setRelation(focusRel->getRel());
	while(!ok){
		if(!red.exec())
			return;
		ok = false;
		QString s = red.relName->text();
		if( (s != focusRel->getRel()->name) && (!relNameUniq(s)) ){
			KMessageBox::sorry(this, "Duplicate relation name", "Error");
			continue;
		}
		ok = true;
	}
	CRelation *rel = focusRel->getRel();
	CTable *from = getTblfromId(rel->fromTbl);
	CTable *to = getTblfromId(rel->toTbl);
	from->delRelation(rel->getId(), rel->fromFld);
	to->delRelation(rel->getId(), rel->toFld);
	rel = red.getRelation();
	getTblfromId(rel->fromTbl)->insertRelation(rel);
	getTblfromId(rel->toTbl)->insertRelation(rel);
	DbDesign *prn = (DbDesign *)(parent());
	prn->ltree->chgRelItem(focusRel->getRel()->name, rel->name);
	focusRel->setRel(rel);
	cnv->setAllChanged();
	cnv->update();
	modified = true;
}

void CView::slotDelRel(){
	if(!focusRel){
		qWarning("Invalid call to slotDelRel() !!");
		return;
	}
	CRelation *rel = focusRel->getRel();
	CTable *from = getTblfromId(rel->fromTbl);
	CTable *to = getTblfromId(rel->toTbl);
	from->delRelation(rel->getId(), rel->fromFld);
	to->delRelation(rel->getId(), rel->toFld);
	DbDesign *main = (DbDesign *)(parent());
	main->ltree->delRelItem(rel->name);
	for(viewlist::Iterator it=views.begin(); it!=views.end(); it++){
		view_map items = views[it.key()];
		items.remove(focusRel->getRel()->getId());
	}
	delete focusRel;
	focusRel = 0;
	cnv->setAllChanged();
	cnv->update();
	modified = true;
}

QString* CView::getTblSql(CTable *table){
	QString *sql = new QString;
	bool didit = false;
	
	*sql += QString("create table ") + QString(table->name()) + QString("(\r\n");
	fldlist l = table->getFields();
	for(fldlist::iterator it = l.begin(); it != l.end(); it++)
		if(it->type_idx < 6){
			if(!didit)
				didit = true;
			else
				*sql += QString(",\r\n");
			*sql += "\t" + it->name + " " + QString(crp.typeName(it->type_idx));
			if(crp.typeName(it->type_idx) == "varchar"){
				QString s1;
				s1.sprintf("%d", it->size);
				*sql += "(" + s1 + ")";
			}
			if(it->flags & FIELD_NOTNULL)
				*sql += QString(" not null ");
			if(it->flags & FIELD_UNIQUE)
				*sql += QString(" unique ");
			if(it->flags & FIELD_PK)
				*sql += QString(" primary key ");
			for(int i=0; i<crp.modifCount(); i++)
				if(it->flags & crp.getModifMask(i))
					*sql += QString(" ") + crp.getModifVal(i) + " ";
			if(it->defval.length() > 0)
				*sql += QString(" default ") + it->defval;
		}
	for(fldlist::iterator it = l.begin(); it != l.end(); it++)
		if(crp.typeName(it->type_idx) == "constraint"){
			*sql += QString(",\r\n\tconstraint ") + it->name + " ";
			if(it->flags & FIELD_PK)
				*sql += QString(" primary key ");
			*sql += "(" + it->defval + ")";
		}
	for(fldlist::iterator it = l.begin(); it != l.end(); it++)
		if(crp.typeName(it->type_idx) == "check"){
			*sql += QString(",\r\n\tcheck (") + it->defval + ")" +" ";
		}
	idlist rels = table->getRelations();
	for(idlist::iterator it = rels.begin(); it != rels.end(); it++){
		CRelation *item = getRelfromId(*it);
		if(item->toTbl == table->getId())
			continue;
		for(fldlist::iterator it1 = l.begin(); it1 != l.end(); it1++)
			if(it1->getId() == item->fromFld){
				*sql += QString(",\r\n\tconstraint ") + item->name + QString(" foreign key (");
				*sql += it1->name + QString(") references ");
				CTable *refdtbl = getTblfromId(item->toTbl);
				*sql += QString(refdtbl->name()) + "(";
				fldlist l1 = refdtbl->getFields();
				for(fldlist::iterator it2 = l1.begin(); it2 != l1.end(); it2++)
					if(it2->getId() == item->toFld){
						*sql += it2->name + ")";
						break;
					}
				break;
			}
	}
	*sql += QString("\r\n);\r\n");
	
	return sql;
}

void CView::slotImportFromFile(){
}

QString* CView::getDbSql(bool drop){
	QString *sql = new QString;
	vector<CTable*> tables;
	hash_map<int, int>  id2idx;
	int graph[MAXTABLES][MAXTABLES];
	bool used[MAXTABLES];
	QCanvasItemList l = cnv->allItems();
	int i, j, k, cnt, n;
	bool found;
	
	*sql = "";
	for(seqlist::iterator it=sequences.begin(); it!=sequences.end(); it++){
		CSequence *seq = (CSequence*)(*it);
		if(drop)
			*sql += QString("drop sequence ") + seq->name + QString(";\r\n");
		QString s;
		*sql += QString("create sequence ") + seq->name;
		s.sprintf("%d", seq->min);
		*sql += QString(" start ") + s;
		s.sprintf("%d", seq->step);
		*sql += QString(" increment ") + s;
		if(seq->max>0){
			s.sprintf("%d", seq->max);
			*sql += QString("maxvalue ") + s;
		}
		*sql += QString(";\r\n\r\n");
	}
	
	for(i=0; i<MAXTABLES; i++)
		for(j=0; j<MAXTABLES; j++)
			graph[i][j] = 0;
	for(i=0; i<MAXTABLES; i++)
		used[i] = 0;
	
	//populate the tables array and establish
	//table id to tables array index relations in id2idx
	i=0;
	tables.clear();
	for(QCanvasItemList::Iterator it = l.begin(); it != l.end(); it++)
		if ( (*it)->rtti() == tableRTTI ){
			CTableItem *item= (CTableItem*)(*it);
			tables.push_back(item->getTable());
			id2idx[item->getTable()->getId()] = i++;
	}
	
	cerr << "\n\n\n";
	//build graph
	n = tables.size();
	for(i=0; i<n; i++){
		idlist rels = tables[i]->getRelations();
		cerr << "Table " << i << " id: " << tables[i]->getId() << "\n";
		cerr << "Table " << i << " relations: \n";
		for(idlist::iterator it = rels.begin(); it != rels.end(); it++){
			CRelation *rel = getRelfromId(*it);
			cerr << "\tFrom: " << rel->fromTbl << "; to: " << rel->toTbl << "\n";
			cerr << "\t\tid2idx[rel->fromTbl]: " << id2idx[rel->fromTbl] << "\n";
			cerr << "\t\tid2idx[rel->toTbl]: " << id2idx[rel->toTbl] << "\n";
			if(rel->toTbl == tables[i]->getId())
				graph[id2idx[rel->fromTbl]][i] = 1;
		}
		cerr << "\n";
	}
	
	cerr << "Number of tables " << n << endl;
	//produce the sql script
	for(k=0; k<n; k++){
		found = false;
		for(i=0; i<n; i++)
			if(!used[i]){
				cnt = 0;
				for(j=0; j<n; j++)
					cnt+=graph[i][j];
				cerr << "Use count for table " << i << " is: " << cnt << "\n\n";
				if(cnt == 0){
					found = true;
					if(drop)
						*sql += QString("drop table ") + tables[i]->name() + QString(";\r\n");
					QString *s = getTblSql(tables[i]);
					*sql += *s + QString("\r\n");
					delete s;
					used[i] = true;
					for(j=0; j<n; j++)
						graph[j][i] = 0;
					break;
				}
			}
		if(!found){
			KMessageBox::sorry(this, "Cannot export: the database contains circular references.", "Error");
			return 0;
		}
	}
	
	cerr << "\n\n\n";
	return sql;
}

void CView::slotExportToFile(){
	QFile f;
	QString *sql;
	
	for(;;){
		QString fnm = KFileDialog::getSaveFileName(QString("::exportdb"), QString("*.sql|SQL scripts\n*|All files"), (QWidget *)parent(), QString("Export database..."));
		if(fnm.right(4) != ".sql")
			fnm += QString(".sql");
		if (fnm.isNull())
			return;
		f.setName(fnm);
		if(!f.exists())
			break;
		else
			if (KMessageBox::questionYesNo(this, "The file exists. Overwrite?") == KMessageBox::Yes)
				break;
	}
	if(f.open(IO_WriteOnly)){
		QTextStream t(&f);
		DbDesign *main = (DbDesign *)(parent());
		sql = getDbSql(main->cfDrop->isChecked());
		if(!sql)
			return;
		t << *sql;
		delete sql;
		f.close();
	}
	else
		KMessageBox::error(this, "Could not write to file.");
}

void CView::slotExportToDb(){
	DbDesign *main = (DbDesign *)(parent());
	QString *sql = getDbSql(main->cfDrop->isChecked());
	if(!sql)
		return;
	CSvrDlg svr(false);
	if(!svr.exec()){
		delete sql;
		return;
	}
	if( !crp.connect_db(svr.svrName->currentText(), svr.portNum->text().toInt(),
			svr.user->text(), svr.pass->text(), svr.dbname->text() ) ){
		KMessageBox::error(this, QString("Could not connect to database server: ") + crp.lastDbError(), "DB server error");
		delete sql;
		return;
	}
	if( !crp.execSQL(sql->latin1()) ){
		KMessageBox::error(this, QString("Could not export database: ") + crp.lastDbError(), "DB server error");
		delete sql;
		return;
	}
	delete sql;
	KMessageBox::information(this, "Export OK", "Database export");
	crp.close_db();
}

QString* CView::getDbXML(QString dbname){
	QString	*xmldata = new QString;
	
	*xmldata += QString("<database>\n");
	*xmldata += QString("\n\t<name>") + dbname +QString("</name>\n");
	*xmldata += QString("\t<author>") + QString("</author>\n");
	*xmldata += QString("\t<plugin>") + crp.name() + QString("</plugin>\n\n");
	
	QCanvasItemList l = cnv->allItems();
	for(QCanvasItemList::Iterator it = l.begin(); it != l.end(); it++)
		if((*it)->rtti() == tableRTTI){
			CTableItem *ti = (CTableItem*)(*it);
			CTable *tbl = ti->getTable();
			*xmldata += QString("\t<table>\n");
			*xmldata += QString("\t\t<name>") + QString(tbl->name()) + QString("</name>\n");
			*xmldata += QString("\t\t<x>") + QString::number((*it)->x()) + QString("</x>\n");
			*xmldata += QString("\t\t<y>") + QString::number((*it)->y()) + QString("</y>\n");
			fldlist flist = tbl->getFields();
			for(fldlist::iterator itf = flist.begin(); itf !=flist.end(); itf++){
				*xmldata += QString("\t\t<field>\n");
				*xmldata += QString("\t\t\t<name>") + itf->name + QString("</name>\n");
				*xmldata += QString("\t\t\t<type>") + QString(crp.typeName(itf->type_idx)) + QString("</type>\n");
				*xmldata += QString("\t\t\t<pkey>");
				if(itf->flags & FIELD_PK)
					*xmldata += "1";
				else
					*xmldata += "0";
				*xmldata += QString("</pkey>\n");
				*xmldata += QString("\t\t\t<unique>");
				if(itf->flags & FIELD_UNIQUE)
					*xmldata += "1";
				else
					*xmldata += "0";
				*xmldata += QString("</unique>\n");
				*xmldata += QString("\t\t\t<notnull>");
				if(itf->flags & FIELD_NOTNULL)
					*xmldata += "1";
				else
					*xmldata += "0";
				*xmldata += QString("</notnull>\n");
				for(int i=0; i<crp.modifCount(); i++)
					if(itf->flags & crp.getModifMask(i))
						*xmldata+= QString("\t\t\t<") + crp.getModifName(i) + "/>\n";
				*xmldata += QString("\t\t\t<size>") + QString::number(itf->size) + QString("</size>\n");
				*xmldata += QString("\t\t\t<default>") + itf->defval + QString("</default>\n");
				*xmldata += QString("\t\t\t<comment>") + itf->defval + QString("</comment>\n");
				*xmldata += QString("\t\t</field>\n");
			}
			*xmldata += QString("\t</table>\n\n");
		}
	for(QCanvasItemList::Iterator it = l.begin(); it != l.end(); it++)
		if((*it)->rtti() == relationRTTI){
			CRelationItem *ri = (CRelationItem*)(*it);
			CRelation *rel = ri->getRel();
			CTable *tbl = getTblfromId(rel->fromTbl);
			CField *fld = tbl->getFldfromId(rel->fromFld);
			
			*xmldata += QString("\t<relation>\n");
			
			*xmldata += QString("\t\t<name>") + rel->name + QString("</name>\n");
			*xmldata += QString("\t\t<fromtbl>") + QString(tbl->name()) + QString("</fromtbl>\n");
			*xmldata += QString("\t\t<fromfld>") + fld->name + QString("</fromfld>\n");
			
			tbl = getTblfromId(rel->toTbl);
			fld = tbl->getFldfromId(rel->toFld);
			*xmldata += QString("\t\t<totbl>") + QString(tbl->name()) + QString("</totbl>\n");
			*xmldata += QString("\t\t<tofld>") + fld->name + QString("</tofld>\n");
			
			*xmldata += QString("\t</relation>\n\n");
		}
		
	for(seqlist::iterator it=sequences.begin(); it!=sequences.end(); it++){
		CSequence *item = (CSequence*)(*it);
		*xmldata += QString("\t<sequence>\n");
		QString s;
		*xmldata += QString("\t\t<name>") + item->name + QString("</name>\n");
		s.sprintf("%d", item->min);
		*xmldata += QString("\t\t<min>") + s + QString("</min>\n");
		s.sprintf("%d", item->max);
		*xmldata += QString("\t\t<max>") + s + QString("</max>\n");
		s.sprintf("%d", item->step);
		*xmldata += QString("\t\t<step>") + s + QString("</step>\n");
		*xmldata += QString("\t</sequence>\n\n");
	}
	
	for(viewlist::Iterator it=views.begin(); it!=views.end(); it++){
		QString name = it.key();
		if(name == "All")
			continue;
		view_map view = it.data();
		
		*xmldata += QString("\t<view>\n");
		*xmldata += QString("\t\t<name>") + name + QString("</name>\n");
		*xmldata += QString("\t\t<items>");
		for(QCanvasItemList::Iterator it = l.begin(); it != l.end(); it++){
			if((*it)->rtti() == tableRTTI){
				CTableItem* item = (CTableItem*)(*it);
				if(view[item->getTable()->getId()])
					*xmldata += QString(item->getTable()->name()) + " ";
			}
			if((*it)->rtti() == relationRTTI){
				CRelationItem* item = (CRelationItem*)(*it);
				if(view[item->getRel()->getId()])
					*xmldata += item->getRel()->name + " ";
			}
		}
		*xmldata += QString("</items>\n");
		*xmldata += QString("\t</view>\n\n");
	}
	
	*xmldata += QString("</database>\n");	
	return xmldata;
}

CTable* CView::getTblfromName(QString name){
	QCanvasItemList l = cnv->allItems();
	for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it)
		if ( (*it)->rtti() == tableRTTI ){
			CTableItem *item = (CTableItem*)(*it);
	    if ( name == item->getTable()->name() )
	    	return item->getTable();
		}
	qWarning("CView::getTblfromName() : returning 0");
	return 0;
}

CRelation* CView::getRelfromName(QString name){
	QCanvasItemList l = cnv->allItems();
	for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it)
		if ( (*it)->rtti() == relationRTTI ){
			CRelationItem *item = (CRelationItem*)(*it);
	    if ( name == item->getRel()->name )
	    	return item->getRel();
		}
	qWarning("CView::getRelfromName() : returning 0");
	return 0;
}

typedef QMap<QString, int> modifmap;

bool CView::loadXML(QFile *f){
	QDomDocument doc;
	int x, y;
	modifmap modifiers;
	
	for(int i=0; i<crp.modifCount(); i++){
		modifiers[crp.getModifName(i)] = crp.getModifMask(i);
	}
//	cerr << modifiers[crp.getModifName(0)] << endl;
	
	if(!doc.setContent(f)){
		cerr << "Something is rotten in the state of Denmarke\n";
		return false;
	}
	QDomElement root = doc.documentElement();
	QDomNode item = root.firstChild();
	while(!item.isNull()){
		QDomElement itemEl = item.toElement();
		if(itemEl.isNull()){
			item = item.nextSibling();
			continue;
		}
		if(itemEl.tagName() == "name")
			((DbDesign *)parent())->setTblName(itemEl.text());
		if(itemEl.tagName() == "author"){
			//not used yet
		}
		if(itemEl.tagName() == "plugin"){
			//TODO!!
		}
		
		/* table parser */
		if(itemEl.tagName() == "table"){
			QDomNode property = item.firstChild();
			x = 0;
			y = 0;
			CTable *table = new CTable(0);
			
			while(!property.isNull()){
				QDomElement propEl = property.toElement();
				if(propEl.isNull()){
					property = property.nextSibling();
					continue;
				}
				if(propEl.tagName() == "x")
					x = propEl.text().toInt();
				if(propEl.tagName() == "y")
					y = propEl.text().toInt();
				if(propEl.tagName() == "name")
					table->setName(propEl.text().latin1());
					
				/* field parser */
				if(propEl.tagName() == "field"){
					QDomNode fieldprop = property.firstChild();
					CField *fld = new CField(0);
					fld->refcnt = 0;
					while(!fieldprop.isNull()){
						QDomElement fieldEl = fieldprop.toElement();
						if(fieldEl.isNull()){
							fieldprop = fieldprop.nextSibling();
							continue;
						}
						if(fieldEl.tagName() == "name")
							fld->name = fieldEl.text();
						if(fieldEl.tagName() == "comment")
							fld->comment = fieldEl.text();
						if(fieldEl.tagName() == "default")
							fld->defval = fieldEl.text();
						if(fieldEl.tagName() == "pkey")
							if(fieldEl.text() == "1")
								fld->flags |= FIELD_PK;
						if(fieldEl.tagName() == "unique")
							if(fieldEl.text() == "1")
								fld->flags |= FIELD_UNIQUE;
						if(fieldEl.tagName() == "notnull")
							if(fieldEl.text() == "1")
								fld->flags |= FIELD_UNIQUE;
						if(fieldEl.tagName() == "size")
							fld->size = fieldEl.text().toInt();
						//if undefined, it's 0, and it won't hurt anyone
						fld->flags |= modifiers[fieldEl.tagName()];
						if(fieldEl.tagName() == "type"){
							int i=0;
							for(i=0; i<crp.typeCount(); i++)
								if(fieldEl.text() == crp.typeName(i))
									break;
							if(i<crp.typeCount())
								fld->type_idx = i;
						}
						fieldprop = fieldprop.nextSibling();
					}
					table->insertField(fld);
				}
				/* end of field parser */
				
				property = property.nextSibling();
			}
			//parsed a table
			if(nrtbl<MAXTABLES){
				if(strlen(table->name()) == 0){
					cerr << "Detected empty table name. User messed with the savefile...\n";
					delete table;
					item = item.nextSibling();
					continue;
				}
				if(table->getFields().size() == 0){
					cerr << "Detected empty table (no fields). User messed with the savefile...\n";
					delete table;
					item = item.nextSibling();
					continue;
				}
				nrtbl++;
				views["All"][table->getId()] = true;
				CTableItem *t = new CTableItem(table, x, y, cnv);
				t->setZ(table->getId());
				t->show();
//				cnv->update();
				DbDesign *prn = (DbDesign *)(parent());
				prn->ltree->insTableLast((const char *)(table->name()), t);
			}
			else{
				cerr << "Too many tables... Possibly sadistic user messed with the savefile...\n";
				delete table;
			}
		}
		/* end of table parser */
		
		/* relation parser */
		if(itemEl.tagName() == "relation"){
			QDomNode property = item.firstChild();
			x = 0;
			y = 0;
			CRelation *rel = new CRelation(0);
			CTable *fromtbl = 0;
			CTable *totbl = 0;
			
			while(!property.isNull()){
				QDomElement propEl = property.toElement();
				if(propEl.isNull()){
					property = property.nextSibling();
					continue;
				}
				if(propEl.tagName() == "name")
					rel->name = propEl.text();
				if(propEl.tagName() == "fromtbl")
					fromtbl = getTblfromName(propEl.text());
				if(propEl.tagName() == "totbl")
					totbl = getTblfromName(propEl.text());
				if(propEl.tagName() == "fromfld")
					if(fromtbl)
						rel->fromFld = fromtbl->getFldfromName(propEl.text())->getId();
					else
						cerr << "Detected from_field declaration before from_table. User messed with the savefile... \n";
				if(propEl.tagName() == "tofld")
					if(totbl)
						rel->toFld = totbl->getFldfromName(propEl.text())->getId();
					else
						cerr << "Detected to_field declaration before to_table. User messed with the savefile... \n";
				property = property.nextSibling();
			}
			if( fromtbl && totbl && rel->toFld && rel->fromFld && (!(rel->name.isNull())) ){
				views["All"][rel->getId()] = true;
				rel->fromTbl = fromtbl->getId();
				rel->toTbl = totbl->getId();
				fromtbl->insertRelation(rel);
				totbl->insertRelation(rel);
				CRelationItem *ri = new CRelationItem(rel, cnv);
				ri->setZ(rel->getId());
				ri->show();
//				cnv->setAllChanged();
//				cnv->update();
				DbDesign *prn = (DbDesign *)(parent());
				prn->ltree->insRelLast(rel->name, ri);
			}
			else{
				cerr << "Bad relation description, ignoring entry\n";
				delete rel;
			}
		}
		/* end of relation parser */
		
		/* sequence parser */
		if(itemEl.tagName() == "sequence"){
			QDomNode property = item.firstChild();
			x = 0;
			y = 0;
			CSequence *seq = new CSequence;
			seq->min = -1;
			seq->max = -1;
			seq->step = -1;
			
			while(!property.isNull()){
				QDomElement propEl = property.toElement();
				if(propEl.isNull()){
					property = property.nextSibling();
					continue;
				}
				if(propEl.tagName() == "name")
					seq->name = propEl.text();
				if(propEl.tagName() == "min")
					seq->min = propEl.text().toInt();
				if(propEl.tagName() == "max")
					seq->max = propEl.text().toInt();
				if(propEl.tagName() == "step")
					seq->step = propEl.text().toInt();
				property = property.nextSibling();
			}
			
			if( (seq->step>=0) && (seq->min>=0) && (seq->max>=0) && (!(seq->name.isNull())) ){
				sequences.push_back(seq);
				DbDesign *prn = (DbDesign *)(parent());
				prn->ltree->insSeqLast(seq);
			}
			else{
				cerr << "Bad sequence description, ignoring entry\n";
				delete seq;
			}
		}
		/* end of sequence parser */
		
		/* view parser */
		if(itemEl.tagName() == "view"){
			QDomNode property = item.firstChild();
			view_map view;
			QString name;
			
			while(!property.isNull()){
				QDomElement propEl = property.toElement();
				if(propEl.isNull()){
					property = property.nextSibling();
					continue;
				}
				if(propEl.tagName() == "name")
					name = propEl.text();
				if(propEl.tagName() == "items"){
					QString items = propEl.text();
					QString it = "";
					for(int i=0; i<items.length(); i++)
						if(items[i]==' '){
							int id = -1;
							CTable *table = getTblfromName(it);
					
							while( (items[i]==' ') && (i<items.length()) )
								i++;
							i-=1;
							if(!table){
								CRelation *rel = getRelfromName(it);
								if(rel)
									id = rel->getId();
							}
							else
								id = table->getId();
							it = "";
							if(id == -1){
								qWarning("Bad item name in view declaration");
								continue;
							}
							view[id] = true;
						}
						else
							it+=items[i];
					if(!it.isEmpty()){
						CTable *table = getTblfromName(it);
						int id = -1;
						if(!table){
							CRelation *rel = getRelfromName(it);
							if(rel)
								id = rel->getId();
						}
						else
							id = table->getId();
						if(id == -1)
							qWarning("Bad item name in view declaration");
						else
							view[id] = true;
					}
				}
				property = property.nextSibling();
			}
			if(!name.isNull())
				views[name] = view;
			else
				qWarning("Bad view declaration: missing name");
		}
		item = item.nextSibling();
	}
	QStringList sl;
	sl.clear();
	for(viewlist::Iterator it=views.begin(); it!=views.end(); it++)
		sl.append(it.key());
	DbDesign *main = (DbDesign *)(parent());
	main->vwView->setItems(sl);
	main->vwView->setCurrentItem(0);
	slotViewChg(QString("All"));
	return true;
}

void CView::slotClear(){
	QCanvasItemList l = cnv->allItems();
	QCanvasItemList::Iterator it;
	for(it = l.begin(); it != l.end(); it++){
		if((*it)->rtti() == relationRTTI){
			focusRel = (CRelationItem*)(*it);
			slotDelRel();
		}
	}
	
	l = cnv->allItems();
	for(it = l.begin(); it != l.end(); it++){
		if((*it)->rtti() == tableRTTI){
			focusItem = (CTableItem*)(*it);
			focusItem->setFocus(true);
			slotDelTbl();
		}
	}
	
	for(seqlist::iterator it=sequences.begin(); it != sequences.end(); it++){
		CSequence *item = (CSequence*)(*it);
		DbDesign *prn = (DbDesign *)(parent());
		prn->ltree->delSeqItem(item->name);
		delete item;
		sequences.erase(it, it);
	}
	
	views.clear();
	view_map all;
	all.clear();
	views["All"] = all;
	crview = "All";
	nrtbl = 0;
}

void CView::slotNewSeq(){
	CSeqEdit seqedit(this);
	bool nameuniq=true;
	bool ok=false;
	
	while(!ok){
		if(!seqedit.exec())
			return;
		ok = false;
		QString s = seqedit.seqname->text();
		for(seqlist::iterator it=sequences.begin(); it!=sequences.end(); it++)
			if(((CSequence*)(*it))->name == s){
				nameuniq=false;
				break;
			}
		if(!nameuniq){
			KMessageBox::sorry(this, "Cannot create 2 sequences with the same name", "Error");
			continue;
		}
		ok = true;
	}
	CSequence* seq = new CSequence;
	seq->name = seqedit.seqname->text();
	seq->min = seqedit.min->value();
	seq->max = seqedit.max->value();
	seq->step = seqedit.step->value();
	sequences.push_back(seq);
	DbDesign *prn = (DbDesign *)(parent());
	prn->ltree->insSeqLast(seq);
	modified = true;
}

void CView::slotEditSeq(){
	CSeqEdit seqedit(this);
	bool nameuniq=true;
	bool ok=false;
	
	seqedit.seqname->setText(focusSeq->name);
	seqedit.min->setValue(focusSeq->min);
	seqedit.max->setValue(focusSeq->max);
	seqedit.step->setValue(focusSeq->step);
	
	while(!ok){
		if(!seqedit.exec())
			return;
		ok = false;
		QString s = seqedit.seqname->text();
		for(seqlist::iterator it=sequences.begin(); it!=sequences.end(); it++)
			if(((CSequence*)(*it))->name == s){
				nameuniq=false;
				break;
			}
		if(!nameuniq){
			KMessageBox::sorry(this, "Cannot create 2 sequences with the same name", "Error");
			continue;
		}
		ok = true;
	}
	DbDesign *prn = (DbDesign *)(parent());
	QString newname=seqedit.seqname->text();
	prn->ltree->chgSeqItem(focusSeq->name, newname);
	focusSeq->name = seqedit.seqname->text();
	focusSeq->min = seqedit.min->value();
	focusSeq->max = seqedit.max->value();
	focusSeq->step = seqedit.step->value();
	modified = true;
}

void CView::slotDelSeq(){
	if(!focusSeq){
		qWarning("Invalid call to slotDelSeq() !!");
		return;
	}
	for(seqlist::iterator it=sequences.begin(); it!=sequences.end(); it++)
		if(((CSequence*)(*it))->name == focusSeq->name){
			sequences.erase(it, it);
			break;
		}
	DbDesign *prn = (DbDesign *)(parent());
	prn->ltree->delSeqItem(focusSeq->name);
	delete focusSeq;
	focusSeq=0;
	modified = true;
}

void CView::slotConfViews(){
	CViewConf dlg;
	item_map items;
	QCanvasItemList l = cnv->allItems();
	
	for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it){
		if ( (*it)->rtti() == tableRTTI ){
			CTableItem *item = (CTableItem*)(*it);
			items[QString(item->getTable()->name())] = item->getTable()->getId();
		}
		if ( (*it)->rtti() == relationRTTI ){
			CRelationItem *item = (CRelationItem*)(*it);
			items[item->getRel()->name] = item->getRel()->getId();
		}
	}
	
	dlg.set_views(views, items);
	if(!dlg.exec())
		return;
	views = dlg.get_views();
	QStringList sl;
	sl.clear();
	int idx, i=0;
	for(viewlist::Iterator it=views.begin(); it!=views.end(); it++, i++){
		sl.append(it.key());
		if(it.key() == dlg.get_current())
			idx = i;
	}
	DbDesign *main = (DbDesign *)(parent());
	main->vwView->setItems(sl);
	main->vwView->setCurrentItem(idx);
	slotViewChg(dlg.get_current());
}

void CView::slotViewChg(QString newv){
	crview = newv;
	QCanvasItemList l = cnv->allItems();
	
	for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it){
		if ( (*it)->rtti() == tableRTTI ){
			CTableItem *item = (CTableItem*)(*it);
			item->visible = views[crview][item->getTable()->getId()];
		}
		if ( (*it)->rtti() == relationRTTI ){
			CRelationItem *item = (CRelationItem*)(*it);
			item->visible = views[crview][item->getRel()->getId()];
		}
	}
	
	cnv->setAllChanged();
	cnv->update();
}
