/***************************************************************************
                          cview.h  -  description
                             -------------------
    begin                : Sat Apr 28 2001
    copyright            : (C) 2001 by Mihnea Balta
    email                : mihnea@tomrad.ro
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CVIEW_H
#define CVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qrect.h>
#include <qwidget.h>
#include <qcanvas.h>
#include <kmessagebox.h>
#include <qpopupmenu.h>
#include <qfile.h>
#include <qmap.h>

#include "ctableitem.h"
#include "crelationitem.h"
#include "ctableed.h"
#include "sequence.h"
#include "viewconf.h"

#include <list>

/****
  @author Mihnea Balta
  CView is the main application view.
  It contains the following objects:
			- CTableItem
			- CRelationItem
	It provides the visual information to the user.
****/

#define MAXTABLES 250

typedef list<CSequence*> seqlist;

class CView : public QCanvasView  {
   Q_OBJECT
public:
  CView(QWidget* parent=0, const char* name=0);
	~CView();
	void					setFocusItem(CTableItem *item);
	void					setFocusRel(CRelationItem *item);
	void					setFocusSeq(CSequence *item);
	CRelation			*getRelfromId(uint);
	CTable				*getTblfromId(uint);
	CTable				*getTblfromName(QString name);
	CRelation			*getRelfromName(QString name);
	CTableItem		*getTblItemfromId(uint);
	CRelationItem	*getRelItemfromId(uint);
	QString				*getDbXML(QString dbname);
	bool					loadXML(QFile *f);
	bool					modified;
	QString				crview;

protected:
	void		contentsMousePressEvent(QMouseEvent*);
	void		contentsMouseMoveEvent(QMouseEvent*);
	void		contentsMouseDoubleClickEvent(QMouseEvent*);

private:
	QCanvas			*cnv;
	CTableItem	*moving;
	CTableItem	*focusItem;
	CRelationItem	*focusRel;
	CSequence		*focusSeq;
	QPoint			moving_start;
	QPopupMenu	*ctxMenu;
	int					menu_id[6];
	QPoint			newTableAt;
	bool				tblNameUniq(QString&);
	bool				relNameUniq(QString&);
	QString*		getTblSql(CTable *table);
	uint				nrtbl;
	QString*		getDbSql(bool drop);
	seqlist			sequences;
	viewlist		views;
	
public slots:
	void		slotNewTbl();
	void		slotEditTbl();
	void		slotDelTbl();
	void		slotImportFromFile();
	void		slotExportToFile();
	void		slotExportToDb();
	void		slotExportTbl();
	void		slotNewRel();
	void		slotEditRel();
	void		slotDelRel();
	void		slotNewSeq();
	void		slotEditSeq();
	void		slotDelSeq();
	void		slotClear();
  void		slotConfViews();
  void		slotViewChg(QString newv);
};

#endif

