/***************************************************************************
                          dbd_plugin.h  -  description
                             -------------------
    begin                : Sun Jun 10 2001
    copyright            : (C) 2001 by Mihnea Balta
    email                : mihnea@tomrad.ro
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef DBD_PLUGIN_H
#define DBD_PLUGIN_H

/* Object affinities */
#define SEQS 0x1

/* Modifier <-> type associations */
#define MODIF_ENABLED 0
#define MODIF_DISABLED 1
#define MODIF_FORCED 2

/* exported functions */

extern "C"{
	/* returns plugin name */
	const char*		getName();
	/* the number of types supported by this plugin */
	int						typecnt();
	/* the type name of passed index */
	const char*		type_name(int);
	/* true if the passed type index can have size */
	bool					has_size(int);
	/* true if the passed type index can be reffered to as a key */
	bool					canbekey(int);
	/* true if references from the first passed type index to the second are valid */
	bool					validref(int, int);
	/* number of plugin-specific modifiers */
	int						modifcnt();
	/* name of modifier pointed by the passed index */
	const char*		modifname(int);
	/* bitmask of modifier pointed by the passed index */
	int						modifmask(int);
	/* how the modifier is represented in the .sql script */
	const char*		modifval(int);
	/* returns one of the MODIF_* constants above, to indicate if the
	   type pointed by the first argument can be added the modifier
	   pointed by the second */
	int						canmodif(int, int);
	/* true if dbdesigner should list fields of this type on the workspace */
	bool					showable(int);
	/* returns a bitmask of features support */
	unsigned int	affinity();
	/* connects to database. syntax:
	   host, port, user, pass, database_name */
	bool					connect(const char*, int, const char*, const char*, const char*);
	/* executes the given sql command and returns true if successful */
	bool					exec(const char*);
	/* terminates connection to backend */
	void					close();
	/* a human-readable string describing the error produced by the last db command;
	   NULL if no error */
	char*					last_error();
	/* returns an array of database names from a server. syntax:
	  host, port, user, pass */
	char**				get_databases(const char*, int, const char*, const char*);
};

#endif
