/***************************************************************************
                          dbdesign.cpp  -  description
                             -------------------
    begin                : Mon Apr  9 18:56:14 EEST 2001
    copyright            : (C) 2001 by Mihnea Balta
    email                : mihnea@tomrad.ro
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dbdesign.h"
#include <iostream>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qdir.h>
#include <kstddirs.h>
#include <stdlib.h>
#include <dlfcn.h>
#include "plugin.h"

extern CPlugin crp;

DbDesign::DbDesign(QWidget *parent, const char *name) : KMainWindow(parent, name)
{
	setAutoSaveSettings();
	gl = new QGridLayout(this, 1, 2);
	gl->setMenuBar(menuBar());
	
  dbview = new CView(this, "dbview");
	ltree = new CObjectTree(this, "ltree");

	gl->addWidget(ltree, 0, 0);
	gl->addWidget(dbview, 0, 1);
	
	gl->activate();
	initAct();
	createGUI();
	dbname = "untitled";
	filename = "";
}

void DbDesign::initAct(){
	QStringList list;
	
	fileNew = KStdAction::openNew(this, SLOT(slotNew()), actionCollection());
	fileNew->setStatusText("New project");
	fileOpen = KStdAction::open(this, SLOT(slotOpen()), actionCollection());
	fileOpen->setStatusText("Opens a saved project");
	fileSave = KStdAction::save(this, SLOT(slotSave()), actionCollection());
	fileSave->setStatusText("Saves the current project");
	fileSaveAs = KStdAction::saveAs(this, SLOT(slotSaveAs()), actionCollection());
	fileSaveAs->setStatusText("Saves the current project under a different name");
	
	fileImport = new KSelectAction("&Import from ...", 0, 0, this, SLOT(slotImport()), actionCollection(),"file_import");	
	fileImport->setStatusText("Imports a project");
	list.append(".&sql script");
	list.append("&database server");
	fileImport->setItems(list);
		
	fileExport = new KSelectAction("E&xport to ...", 0, 0, this, SLOT(slotExport()), actionCollection(),"file_export");	
	fileExport->setStatusText("Exports the project");
	fileExport->setItems(list);
	
	fileQuit = KStdAction::quit(this, SLOT(slotQuit()), actionCollection());
	fileQuit->setStatusText("Quits DbDesign");
	objCrTable = new KAction("Create &Table", 0, 0, dbview, SLOT(slotNewTbl()), actionCollection(),"obj_CrTable");	
	objCrTable->setStatusText("Creates a new table");
	objCrTable->setAccel(Key_T);
	objCrRel = new KAction("Create &Relation", 0, 0, dbview, SLOT(slotNewRel()), actionCollection(),"obj_CrRel");	
	objCrRel->setStatusText("Creates a new relation");
	objCrRel->setAccel(Key_R);
	editEditTable = new KAction("&Edit Table", 0, 0, dbview, SLOT(slotEditTbl()), actionCollection(),"edit_EditTable");
	editEditTable->setStatusText("Edits the selected table");
	editEditTable->setAccel(Key_E);
	editDelTable = new KAction("&Delete Table", 0, 0, dbview, SLOT(slotDelTbl()), actionCollection(),"edit_DelTable");
	editDelTable->setStatusText("Deletes the selected table");
	editDelTable->setAccel(Key_Delete);
	editClear = new KAction("&Clear workspace", 0, 0, dbview, SLOT(slotClear()), actionCollection(),"edit_Clear");
	editClear->setStatusText("Deletes everything from the workspace");
	
	cfPlugin = new KSelectAction("&Select plug-in", 0, 0, this, SLOT(slotPlugin()), actionCollection(),"cf_Plugin");	
	cfPlugin->setStatusText("Selects a plug-in");
	cfDrop = new KToggleAction("&Drop schema first", 0, 0, 0, 0, actionCollection(), "cf_Drop");
	
	vwConf = new KAction("&Configure views", 0, 0, dbview, SLOT(slotConfViews()), actionCollection(),"view_conf");
	vwSep = new KActionSeparator(actionCollection(), "view_sep");
	vwView = new KSelectAction("&Select view", 0, 0, this, SLOT(slotView()), actionCollection(),"view_view");	
	list.clear();
	list.append("All");
	vwView->setItems(list);
	vwView->setCurrentItem(0);
	
	if(refreshPlugins() != 0){
		KMessageBox::error(0, "Cannot start: unable to find the 'Standard' plugin.", "Error");
		exit(1);
	}
	crp.load(plugins[stdplugidx].path);
	if(!crp.ok()){
		KMessageBox::error(0, "Cannot start: unable to load the 'Standard' plugin.", "Error");
		exit(1);
	}
}

DbDesign::~DbDesign(){
}

void DbDesign::slotNew(){
	if(dbview->modified){
		switch(KMessageBox::warningYesNoCancel(this, "The project was modified. Save changes?", "Question")){
			case KMessageBox::Yes:
				slotSave();
				break;
			case KMessageBox::No:
				break;
			case KMessageBox::Cancel:
				return;
		}
	}
	dbname = "untitled";
	filename = "";
	dbview->slotClear();
	dbview->modified = false;
}

void DbDesign::slotOpen(){
	if(dbview->modified){
		switch(KMessageBox::warningYesNoCancel(this, "The project was modified. Save changes?", "Question")){
			case KMessageBox::Yes:
				slotSave();
				break;
			case KMessageBox::No:
				break;
			case KMessageBox::Cancel:
				return;
		}
	}
	
	QFile f;
	QString fnm = KFileDialog::getOpenFileName(QString("::opendb"), QString("*.dbd|DbDesigner files\n*|All files"), this);
	
	if(fnm.isEmpty() || fnm.isNull())
		return;
	dbview->slotClear();
	f.setName(fnm);
	if(f.open(IO_ReadOnly)){
		if(!dbview->loadXML(&f))
			KMessageBox::error(this, "Not a valid dbdesigner file.");
		f.close();
		dbview->modified = false;
		filename = fnm;
	}
	else
		KMessageBox::error(this, "Could not open file.");
}

void DbDesign::slotSave(){
	if(filename.isEmpty() || filename.isNull()){
		slotSaveAs();
		return;
	}
	
	QString* xmldata = dbview->getDbXML(dbname);
	QFile f;
	f.setName(filename);
	if(f.open(IO_WriteOnly)){
		QTextStream t(&f);
		t << *xmldata;
		f.close();
		dbview->modified = false;
	}
	else
		KMessageBox::error(this, "Could not write to file.");
	delete xmldata;
}

void DbDesign::slotSaveAs(){
	QFile f;
	QString fnm;
	
	for(;;){
		fnm = KFileDialog::getSaveFileName(QString("::saveas"), QString("*.dbd|DbDesigner files\n*|All files"), this);
		if (fnm.isEmpty() || fnm.isNull())
			return;
		if(fnm.right(4) != ".dbd")
			fnm += QString(".dbd");
		f.setName(fnm);
		if(!f.exists())
			break;
		else
			if (KMessageBox::questionYesNo(this, "The file exists. Overwrite?") == KMessageBox::Yes)
				break;
	}
	filename = fnm;
	slotSave();
}

void DbDesign::slotQuit(){
	if(dbview->modified){
		switch(KMessageBox::warningYesNoCancel(this, "The project was modified. Save changes?", "Question")){
			case KMessageBox::Yes:
				slotSave();
				break;
			case KMessageBox::No:
				break;
			case KMessageBox::Cancel:
				return;
		}
	}
	close();
}

void DbDesign::slotImport(){
}

void DbDesign::slotExport(){
	int i = fileExport->currentItem();
	//the following line is a hack, because for some reason
	//KListAction calls the slot twice upon item selection and
	//thus i couldn't use it.
	fileExport->setCurrentItem(-1);
	switch(i){
		case 0:
			dbview->slotExportToFile();
			break;
		case 1:
			dbview->slotExportToDb();
			break;
	}
}

void DbDesign::setTblName(QString newname){
	dbname = newname.latin1();
}

bool DbDesign::queryClose(){
	if(dbview->modified){
		switch(KMessageBox::warningYesNoCancel(this, "The project was modified. Save changes?", "Question")){
			case KMessageBox::Yes:
				slotSave();
				if(dbview->modified)
					return false;
				else
					return true;
			case KMessageBox::No:
				return true;
			case KMessageBox::Cancel:
				return false;
		}
	}
	return true;
}

void DbDesign::slotPlugin(){
}

/*
	return values:
		0 - ok
		1 - no files found
		2 - standard plugin not found
*/

int DbDesign::refreshPlugins(){
	QString path = QString(getenv("KDEDIR")) + "/" + KStandardDirs::kde_default("data") + "/dbdesign/plugins/";
	QDir pldir(path, "*plug.so", QDir::IgnoreCase | QDir::Name, QDir::Files);
	bool foundstd = false;
	void* h;
	void* func;
	QStringList menu;
	
	plugins.clear();
	if(pldir.count() < 1)
		return 1;
	for(int i=0; i<pldir.count(); i++){
		QString fname = path + pldir[i];
		h = dlopen(fname.latin1(), RTLD_NOW);
		if(h == NULL){
			cerr << "Could not dlopen() '" << fname << "', skipping.\n";
			continue;
		}
		func = dlsym(h, "getName");
		if(func == NULL){
			cerr << "No getName() symbol for '" << fname << "', skipping.\n";
			continue;
		}
		const char *pname = ((const char * (*)())func)();
		QString nm(pname);
		PlugInfo *pi = new PlugInfo;
		pi->name = nm;
		pi->path = fname;
		plugins.push_back(*pi);
		menu.append(QString(pname));
		dlclose(h);
		
		if(nm == "Standard"){
			stdplugidx = i;
			foundstd = true;
		}
	}
	cfPlugin->setItems(menu);
	cfPlugin->setCurrentItem(stdplugidx);
	if(!foundstd)
		return 2;
	else
		return 0;
}

void DbDesign::slotView(){
	dbview->slotViewChg(vwView->currentText());
}
