/***************************************************************************
                          plugin.cpp  -  description
                             -------------------
    begin                : Sat May 26 2001
    copyright            : (C) 2001 by Mihnea Balta
    email                : mihnea@tomrad.ro
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "plugin.h"
#include <dlfcn.h>
#include <iostream>

CPlugin::CPlugin(){
	initOk = false;
	handle = 0;
}

CPlugin::~CPlugin(){
	if(handle)
		dlclose(handle);
}

CPlugin::CPlugin(QString fname){
	initOk = false;
	handle = 0;
	load(fname);
}
	
void CPlugin::load(QString fname){
	void *func;
		
	if(handle){
		dlclose(handle);
		cerr << "previous handle found, needed dlclose()\n";
	}
	handle = dlopen(fname.latin1(), RTLD_NOW);
	if(!handle){
		initOk = false;
		return;
	}
	/* name() */
	func = dlsym(handle, "getName");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	getname_ptr = (const char* (*)())func;
	/* typeCount() */
	func = dlsym(handle, "typecnt");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	typecnt_ptr = (int (*)())func;
	/* typeName() */
	func = dlsym(handle, "type_name");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	typename_ptr = (const char* (*)(int))func;
	/* hasSize() */
	func = dlsym(handle, "has_size");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	has_size_ptr = (bool (*)(int))func;
	/* canbekey() */
	func = dlsym(handle, "canbekey");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	canbekey_ptr = (bool (*)(int))func;
	/* validref() */
	func = dlsym(handle, "validref");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	validref_ptr = (bool (*)(int, int))func;
	/* modifCount() */
	func = dlsym(handle, "modifcnt");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	modifcnt_ptr = (int (*)())func;
	/* getModifName() */
	func = dlsym(handle, "modifname");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	modifname_ptr = (const char * (*)(int))func;
	/* getModifMask() */
	func = dlsym(handle, "modifmask");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	modifmask_ptr = (int (*)(int))func;
	/* getModifVal() */
	func = dlsym(handle, "modifval");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	modifval_ptr = (const char * (*)(int))func;
	/* canModif() */
	func = dlsym(handle, "canmodif");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	canmodif_ptr = (int (*)(int, int))func;
	/* showable() */
	func = dlsym(handle, "showable");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	showable_ptr = (bool (*)(int))func;
	/* objAffinity() */
	func = dlsym(handle, "affinity");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	affinity_ptr = (unsigned int (*)())func;
	/* connect_db */
	func = dlsym(handle, "connect");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	connect_ptr = (bool (*)(const char*, int, const char*, const char*, const char*))func;
	/* execSQL() */
	func = dlsym(handle, "exec");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	exec_ptr = (bool (*)(const char*))func;
	/* close_db() */
	func = dlsym(handle, "close");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	close_ptr = (void (*)())func;
	/* lastDbError() */
	func = dlsym(handle, "last_error");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	last_error_ptr = (char* (*)())func;
	/* getDbNames() */
	func = dlsym(handle, "get_databases");
	if(!func){
		initOk = false;
		dlclose(handle);
		return;
	}
	get_databases_ptr = (char** (*)(const char*, int, const char*, const char*))func;
	
	initOk = true;
}

QString CPlugin::name(){
	return QString(getname_ptr());
}

int CPlugin::typeCount(){
	if(initOk)
		return typecnt_ptr();
	else
		return 0;
}

const char* CPlugin::typeName(int type_idx){
	if(initOk)
		return typename_ptr(type_idx);
	else
		return 0;
}

bool CPlugin::hasSize(int type_idx){
	if(initOk)
		return has_size_ptr(type_idx);
	else
		return false;
}

bool CPlugin::canbekey(int type_idx){
	if(initOk)
		return canbekey_ptr(type_idx);
	else
		return false;
}

bool CPlugin::validref(int type1, int type2){
	if(initOk)
		return validref_ptr(type1, type2);
	else
		return false;
}

int CPlugin::modifCount(){
	if(initOk)
		return modifcnt_ptr();
	else
		return 0;
}

QString CPlugin::getModifName(int idx){
	if(initOk)
		return QString(modifname_ptr(idx));
	else
		return 0;
}
	
int CPlugin::getModifMask(int idx){
	if(initOk)
		return modifmask_ptr(idx);
	else
		return 0;
}

const char* CPlugin::getModifVal(int idx){
	if(initOk)
		return modifval_ptr(idx);
	else
		return 0;
}

int CPlugin::canModif(int type_idx, int modif_idx){
	return canmodif_ptr(type_idx, modif_idx);
}

bool CPlugin::showable(int type_idx){
	if(initOk)
		return showable_ptr(type_idx);
	else
		return false;
}

unsigned int CPlugin::objAffinity(){
	if(initOk)
		return affinity_ptr();
	else
		return 0;
}

bool CPlugin::connect_db(QString host, int port, QString user, QString pass, QString db){
	if(initOk)
		return connect_ptr(host.latin1(), port, user.latin1(), pass.latin1(), db.latin1());
	else
		return false;
}

bool CPlugin::execSQL(QString sql){
	if(initOk)
		return exec_ptr(sql.latin1());
	else
		return false;
}

void CPlugin::close_db(){
	if(initOk)
		close_ptr();
}

const bool CPlugin::ok(){
	return initOk;
}

char* CPlugin::lastDbError(){
	return last_error_ptr();
}

char** CPlugin::getDbNames(QString host, int port, QString user, QString pass){
	return get_databases_ptr(host.latin1(), port, user.latin1(), pass.latin1());
}
